@@ -1,20 +1,20 @@
 """The initialization file for the Pywikibot framework."""
 #
-# (C) Pywikibot team, 2008-2021
+# (C) Pywikibot team, 2008-2022
 #
 # Distributed under the terms of the MIT license.
 #
 import atexit
 import datetime
-import inspect
 import math
 import re
+import sys
 import threading
 import time
 from contextlib import suppress
 from decimal import Decimal
 from queue import Queue
-from typing import Optional, Union
+from typing import Any, Optional, Type, Union
 from urllib.parse import urlparse
 from warnings import warn
 
@@ -32,7 +32,14 @@ from pywikibot.__metadata__ import (
     __version__,
 )
 from pywikibot._wbtypes import WbRepresentation as _WbRepresentation
-from pywikibot.backports import cache, removesuffix
+from pywikibot.backports import (  # skipcq: PY-W2000
+    Callable,
+    Dict,
+    List,
+    Tuple,
+    cache,
+    removesuffix,
+)
 from pywikibot.bot import (
     Bot,
     CurrentPageBot,
@@ -46,10 +53,6 @@ from pywikibot.bot import (
     ui,
 )
 from pywikibot.diff import PatchManager
-from pywikibot.exceptions import (
-    DEPRECATED_EXCEPTIONS,
-    CoordinateGlobeUnknownError,
-)
 from pywikibot.family import AutoFamily, Family
 from pywikibot.i18n import translate
 from pywikibot.logging import (
@@ -57,45 +60,38 @@ from pywikibot.logging import (
     debug,
     error,
     exception,
+    info,
     log,
     output,
     stdout,
     warning,
 )
-from pywikibot.site import APISite, BaseSite, ClosedSite, DataSite
-from pywikibot.tools import (
-    ModuleDeprecationWrapper as _ModuleDeprecationWrapper,
-)
-from pywikibot.tools import classproperty
-from pywikibot.tools import deprecate_arg as _deprecate_arg
-from pywikibot.tools import normalize_username
-from pywikibot.tools.formatter import color_format
+from pywikibot.site import APISite, BaseSite, DataSite
+from pywikibot.tools import classproperty, normalize_username
 
 
+ItemPageStrNoneType = Union[str, 'ItemPage', None]
+ToDecimalType = Union[int, float, str, 'Decimal', None]
+
 __all__ = (
     '__copyright__', '__description__', '__download_url__', '__license__',
-    '__maintainer__', '__maintainer_email__', '__name__',
-    '__url__', '__version__',
-    'Bot', 'calledModuleName', 'CaptchaError', 'CascadeLockedPage',
-    'Category', 'CircularRedirect', 'Claim', 'Coordinate',
-    'CoordinateGlobeUnknownException', 'critical', 'CurrentPageBot', 'debug',
-    'EditConflict', 'error', 'Error', 'exception', 'FatalServerError',
-    'FilePage', 'handle_args', 'html2unicode', 'input', 'input_choice',
-    'input_yn', 'InterwikiRedirectPage', 'InvalidTitle', 'IsNotRedirectPage',
-    'IsRedirectPage', 'ItemPage', 'Link', 'LockedNoPage', 'LockedPage', 'log',
-    'NoCreateError', 'NoMoveTarget', 'NoPage', 'NoUsername',
-    'NoWikibaseEntity', 'OtherPageSaveError', 'output', 'Page',
-    'PageCreatedConflict', 'PageDeletedConflict', 'PageRelatedError',
-    'PageSaveRelatedError', 'PropertyPage', 'SectionError', 'Server414Error',
-    'Server504Error', 'ServerError', 'showDiff', 'show_help', 'Site',
-    'SiteDefinitionError', 'SiteLink', 'SpamblacklistError', 'stdout',
-    'Timestamp', 'TitleblacklistError', 'translate', 'ui', 'unicode2html',
-    'UnknownExtension', 'UnknownFamily', 'UnknownSite', 'UnsupportedPage',
-    'UploadWarning', 'url2unicode', 'User', 'warning', 'WbGeoShape',
-    'WbMonolingualText', 'WbQuantity', 'WbTabularData', 'WbTime', 'WbUnknown',
-    'WikiBaseError', 'WikidataBot',
+    '__maintainer__', '__maintainer_email__', '__name__', '__url__',
+    '__version__',
+    'Bot', 'calledModuleName', 'Category', 'Claim', 'Coordinate', 'critical',
+    'CurrentPageBot', 'debug', 'error', 'exception', 'FilePage', 'handle_args',
+    'html2unicode', 'info', 'input', 'input_choice', 'input_yn', 'ItemPage',
+    'LexemeForm', 'LexemePage', 'LexemeSense', 'Link', 'log', 'MediaInfo',
+    'output', 'Page', 'PropertyPage', 'showDiff', 'show_help', 'Site',
+    'SiteLink', 'stdout', 'Timestamp', 'translate', 'ui', 'url2unicode',
+    'User', 'warning', 'WbGeoShape', 'WbMonolingualText', 'WbQuantity',
+    'WbTabularData', 'WbTime', 'WbUnknown', 'WikidataBot',
 )
 
+# argvu is set by pywikibot.bot when it's imported
+
+if not hasattr(sys.modules[__name__], 'argvu'):
+    argvu = []  # type: List[str]
+
 
 class Timestamp(datetime.datetime):
 
@@ -120,17 +116,17 @@ class Timestamp(datetime.datetime):
     mediawikiTSFormat = '%Y%m%d%H%M%S'
     _ISO8601Format_new = '{0:+05d}-{1:02d}-{2:02d}T{3:02d}:{4:02d}:{5:02d}Z'
 
-    def clone(self):
+    def clone(self) -> datetime.datetime:
         """Clone this instance."""
         return self.replace(microsecond=self.microsecond)
 
     @classproperty
-    def ISO8601Format(cls):
+    def ISO8601Format(cls: Type['Timestamp']) -> str:
         """ISO8601 format string class property for compatibility purpose."""
         return cls._ISO8601Format()
 
     @classmethod
-    def _ISO8601Format(cls, sep: str = 'T') -> str:
+    def _ISO8601Format(cls: Type['Timestamp'], sep: str = 'T') -> str:
         """ISO8601 format string.
 
         :param sep: one-character separator, placed between the date and time
@@ -140,14 +136,13 @@ class Timestamp(datetime.datetime):
         return '%Y-%m-%d{}%H:%M:%SZ'.format(sep)
 
     @classmethod
-    def fromISOformat(cls, ts, sep: str = 'T'):
+    def fromISOformat(cls: Type['Timestamp'], ts: Union[str, 'Timestamp'],
+                      sep: str = 'T') -> 'Timestamp':
         """Convert an ISO 8601 timestamp to a Timestamp object.
 
         :param ts: ISO 8601 timestamp or a Timestamp object already
-        :type ts: str or Timestamp
         :param sep: one-character separator, placed between the date and time
         :return: Timestamp object
-        :rtype: Timestamp
         """
         # If inadvertently passed a Timestamp object, use replace()
         # to create a clone.
@@ -156,7 +151,8 @@ class Timestamp(datetime.datetime):
         return cls.strptime(ts, cls._ISO8601Format(sep))
 
     @classmethod
-    def fromtimestampformat(cls, ts):
+    def fromtimestampformat(cls: Type['Timestamp'], ts: Union[str, 'Timestamp']
+                            ) -> 'Timestamp':
         """Convert a MediaWiki internal timestamp to a Timestamp object."""
         # If inadvertently passed a Timestamp object, use replace()
         # to create a clone.
@@ -166,7 +162,7 @@ class Timestamp(datetime.datetime):
             ts += '000'
         return cls.strptime(ts, cls.mediawikiTSFormat)
 
-    def isoformat(self, sep='T'):
+    def isoformat(self, sep: str = 'T') -> str:  # type: ignore[override]
         """
         Convert object to an ISO 8601 timestamp accepted by MediaWiki.
 
@@ -176,15 +172,15 @@ class Timestamp(datetime.datetime):
         """
         return self.strftime(self._ISO8601Format(sep))
 
-    def totimestampformat(self):
+    def totimestampformat(self) -> str:
         """Convert object to a MediaWiki internal timestamp."""
         return self.strftime(self.mediawikiTSFormat)
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Return a string format recognized by the API."""
         return self.isoformat()
 
-    def __add__(self, other):
+    def __add__(self, other: datetime.timedelta) -> 'Timestamp':
         """Perform addition, returning a Timestamp instead of datetime."""
         newdt = super().__add__(other)
         if isinstance(newdt, datetime.datetime):
@@ -193,7 +189,8 @@ class Timestamp(datetime.datetime):
                              newdt.tzinfo)
         return newdt
 
-    def __sub__(self, other):
+    def __sub__(self, other: datetime.timedelta  # type: ignore[override]
+                ) -> 'Timestamp':
         """Perform subtraction, returning a Timestamp instead of datetime."""
         newdt = super().__sub__(other)
         if isinstance(newdt, datetime.datetime):
@@ -209,19 +206,19 @@ class Coordinate(_WbRepresentation):
 
     _items = ('lat', 'lon', 'entity')
 
-    @_deprecate_arg('entity', 'globe_item')
-    def __init__(self, lat: float, lon: float, alt=None,
+    def __init__(self, lat: float, lon: float, alt: Optional[float] = None,
                  precision: Optional[float] = None,
                  globe: Optional[str] = None, typ: str = '',
                  name: str = '', dim: Optional[int] = None,
-                 site: Optional[DataSite] = None, globe_item=None,
-                 primary: bool = False):
+                 site: Optional[DataSite] = None,
+                 globe_item: ItemPageStrNoneType = None,
+                 primary: bool = False) -> None:
         """
         Represent a geo coordinate.
 
         :param lat: Latitude
         :param lon: Longitude
-        :param alt: Altitude? TODO FIXME
+        :param alt: Altitude
         :param precision: precision
         :param globe: Which globe the point is on
         :param typ: The type of coordinate point
@@ -231,7 +228,6 @@ class Coordinate(_WbRepresentation):
         :param globe_item: The Wikibase item for the globe, or the entity URI
                            of this Wikibase item. Takes precedence over 'globe'
                            if present.
-        :type globe_item: pywikibot.ItemPage or str
         :param primary: True for a primary set of coordinates
         """
         self.lat = lat
@@ -252,11 +248,11 @@ class Coordinate(_WbRepresentation):
         self.globe = globe
 
     @property
-    def entity(self):
+    def entity(self) -> str:
         """Return the entity uri of the globe."""
         if not self._entity:
             if self.globe not in self.site.globes():
-                raise CoordinateGlobeUnknownError(
+                raise exceptions.CoordinateGlobeUnknownError(
                     '{} is not supported in Wikibase yet.'
                     .format(self.globe))
             return self.site.globes()[self.globe]
@@ -266,7 +262,7 @@ class Coordinate(_WbRepresentation):
 
         return self._entity
 
-    def toWikibase(self) -> dict:
+    def toWikibase(self) -> Dict[str, Any]:
         """
         Export the data to a JSON object for the Wikibase API.
 
@@ -282,21 +278,21 @@ class Coordinate(_WbRepresentation):
                 }
 
     @classmethod
-    def fromWikibase(cls, data: dict, site: DataSite):
+    def fromWikibase(cls: Type['Coordinate'], data: Dict[str, Any],
+                     site: Optional[DataSite] = None) -> 'Coordinate':
         """
         Constructor to create an object from Wikibase's JSON output.
 
         :param data: Wikibase JSON
         :param site: The Wikibase site
-        :rtype: Coordinate
         """
+        if site is None:
+            site = Site().data_repository()
+
         globe = None
 
         if data['globe']:
-            globes = {}
-            for name, entity in site.globes().items():
-                globes[entity] = name
-
+            globes = {entity: name for name, entity in site.globes().items()}
             globe = globes.get(data['globe'])
 
         return cls(data['latitude'], data['longitude'],
@@ -341,7 +337,7 @@ class Coordinate(_WbRepresentation):
         return self._precision
 
     @precision.setter
-    def precision(self, value):
+    def precision(self, value: float) -> None:
         self._precision = value
 
     def precisionToDim(self) -> Optional[int]:
@@ -380,7 +376,7 @@ class Coordinate(_WbRepresentation):
         return self._dim
 
     def get_globe_item(self, repo: Optional[DataSite] = None,
-                       lazy_load: bool = False):
+                       lazy_load: bool = False) -> 'ItemPage':
         """
         Return the ItemPage corresponding to the globe.
 
@@ -440,7 +436,7 @@ class WbTime(_WbRepresentation):
                  after: int = 0,
                  timezone: int = 0,
                  calendarmodel: Optional[str] = None,
-                 site: Optional[DataSite] = None):
+                 site: Optional[DataSite] = None) -> None:
         """Create a new WbTime object.
 
         The precision can be set by the Wikibase int value (0-14) or by a human
@@ -515,19 +511,20 @@ class WbTime(_WbRepresentation):
                     and precision in self.PRECISION.values()):
                 self.precision = precision
             elif precision in self.PRECISION:
+                assert isinstance(precision, str)
                 self.precision = self.PRECISION[precision]
             else:
                 raise ValueError('Invalid precision: "{}"'.format(precision))
 
     @classmethod
-    def fromTimestr(cls,
+    def fromTimestr(cls: Type['WbTime'],
                     datetimestr: str,
                     precision: Union[int, str] = 14,
                     before: int = 0,
                     after: int = 0,
                     timezone: int = 0,
                     calendarmodel: Optional[str] = None,
-                    site: Optional[DataSite] = None):
+                    site: Optional[DataSite] = None) -> 'WbTime':
         """Create a new WbTime object from a UTC date/time string.
 
         The timestamp differs from ISO 8601 in that:
@@ -547,7 +544,6 @@ class WbTime(_WbRepresentation):
         :param timezone: Timezone information in minutes.
         :param calendarmodel: URI identifying the calendar model
         :param site: The Wikibase site
-        :rtype: pywikibot.WbTime
         """
         match = re.match(r'([-+]?\d+)-(\d+)-(\d+)T(\d+):(\d+):(\d+)Z',
                          datetimestr)
@@ -559,15 +555,15 @@ class WbTime(_WbRepresentation):
                    precision, before, after, timezone, calendarmodel, site)
 
     @classmethod
-    def fromTimestamp(cls, timestamp, precision: Union[int, str] = 14,
+    def fromTimestamp(cls: Type['WbTime'], timestamp: 'Timestamp',
+                      precision: Union[int, str] = 14,
                       before: int = 0, after: int = 0,
                       timezone: int = 0, calendarmodel: Optional[str] = None,
-                      site: Optional[DataSite] = None):
+                      site: Optional[DataSite] = None) -> 'WbTime':
         """
         Create a new WbTime object from a pywikibot.Timestamp.
 
         :param timestamp: Timestamp
-        :type timestamp: pywikibot.Timestamp
         :param precision: The unit of the precision of the time.
         :param before: Number of units after the given time it could be, if
             uncertain. The unit is given by the precision.
@@ -576,7 +572,6 @@ class WbTime(_WbRepresentation):
         :param timezone: Timezone information in minutes.
         :param calendarmodel: URI identifying the calendar model
         :param site: The Wikibase site
-        :rtype: pywikibot.WbTime
         """
         return cls.fromTimestr(timestamp.isoformat(), precision=precision,
                                before=before, after=after,
@@ -612,7 +607,7 @@ class WbTime(_WbRepresentation):
         return Timestamp.fromISOformat(
             self.toTimestr(force_iso=True).lstrip('+'))
 
-    def toWikibase(self) -> dict:
+    def toWikibase(self) -> Dict[str, Any]:
         """
         Convert the data to a JSON object for the Wikibase API.
 
@@ -628,17 +623,17 @@ class WbTime(_WbRepresentation):
         return json
 
     @classmethod
-    def fromWikibase(cls, wb: dict, site: Optional[DataSite] = None):
+    def fromWikibase(cls: Type['WbTime'], data: Dict[str, Any],
+                     site: Optional[DataSite] = None) -> 'WbTime':
         """
         Create a WbTime from the JSON data given by the Wikibase API.
 
-        :param wb: Wikibase JSON
+        :param data: Wikibase JSON
         :param site: The Wikibase site
-        :rtype: pywikibot.WbTime
         """
-        return cls.fromTimestr(wb['time'], wb['precision'],
-                               wb['before'], wb['after'],
-                               wb['timezone'], wb['calendarmodel'], site)
+        return cls.fromTimestr(data['time'], data['precision'],
+                               data['before'], data['after'],
+                               data['timezone'], data['calendarmodel'], site)
 
 
 class WbQuantity(_WbRepresentation):
@@ -648,7 +643,7 @@ class WbQuantity(_WbRepresentation):
     _items = ('amount', 'upperBound', 'lowerBound', 'unit')
 
     @staticmethod
-    def _require_errors(site: DataSite) -> bool:
+    def _require_errors(site: Optional[DataSite]) -> bool:
         """
         Check if Wikibase site is so old it requires error bounds to be given.
 
@@ -664,7 +659,7 @@ class WbQuantity(_WbRepresentation):
         return site.mw_version < '1.29.0-wmf.2'
 
     @staticmethod
-    def _todecimal(value: str) -> Optional[Decimal]:
+    def _todecimal(value: ToDecimalType) -> Optional[Decimal]:
         """
         Convert a string to a Decimal for use in WbQuantity.
 
@@ -679,7 +674,7 @@ class WbQuantity(_WbRepresentation):
         return Decimal(str(value))
 
     @staticmethod
-    def _fromdecimal(value: Decimal) -> Optional[str]:
+    def _fromdecimal(value: Optional[Decimal]) -> Optional[str]:
         """
         Convert a Decimal to a string representation suitable for WikiBase.
 
@@ -687,24 +682,20 @@ class WbQuantity(_WbRepresentation):
 
         :param value: decimal number to convert
         """
-        if value is None:
-            return None
-        return format(value, '+g')
+        return format(value, '+g') if value is not None else None
 
-    def __init__(self, amount, unit=None, error=None,
-                 site: Optional[DataSite] = None):
+    def __init__(self, amount: ToDecimalType,
+                 unit: ItemPageStrNoneType = None,
+                 error: Union[ToDecimalType,
+                              Tuple[ToDecimalType, ToDecimalType]] = None,
+                 site: Optional[DataSite] = None) -> None:
         """
         Create a new WbQuantity object.
 
         :param amount: number representing this quantity
-        :type amount: str or Decimal. Other types are accepted, and
-            converted via str to Decimal.
         :param unit: the Wikibase item for the unit or the entity URI of this
             Wikibase item.
-        :type unit: pywikibot.ItemPage, str or None
         :param error: the uncertainty of the amount (e.g. ±1)
-        :type error: same as amount, or tuple of two values, where the first
-            value is the upper error and the second is the lower error value.
         :param site: The Wikibase site
         """
         if amount is None:
@@ -723,25 +714,28 @@ class WbQuantity(_WbRepresentation):
             self.upperBound = self.lowerBound = None
         else:
             if error is None:
-                upperError = lowerError = Decimal(0)
+                upperError = lowerError = Decimal(0)  # type: Optional[Decimal]
             elif isinstance(error, tuple):
                 upperError = self._todecimal(error[0])
                 lowerError = self._todecimal(error[1])
             else:
                 upperError = lowerError = self._todecimal(error)
 
+            assert upperError is not None and lowerError is not None
+            assert self.amount is not None
+
             self.upperBound = self.amount + upperError
             self.lowerBound = self.amount - lowerError
 
     @property
-    def unit(self):
+    def unit(self) -> str:
         """Return _unit's entity uri or '1' if _unit is None."""
         if isinstance(self._unit, ItemPage):
             return self._unit.concept_uri()
         return self._unit or '1'
 
     def get_unit_item(self, repo: Optional[DataSite] = None,
-                      lazy_load: bool = False):
+                      lazy_load: bool = False) -> 'ItemPage':
         """
         Return the ItemPage corresponding to the unit.
 
@@ -763,7 +757,7 @@ class WbQuantity(_WbRepresentation):
         self._unit = ItemPage.from_entity_uri(repo, self._unit, lazy_load)
         return self._unit
 
-    def toWikibase(self) -> dict:
+    def toWikibase(self) -> Dict[str, Any]:
         """
         Convert the data to a JSON object for the Wikibase API.
 
@@ -777,25 +771,25 @@ class WbQuantity(_WbRepresentation):
         return json
 
     @classmethod
-    def fromWikibase(cls, wb: dict, site: Optional[DataSite] = None):
+    def fromWikibase(cls: Type['WbQuantity'], data: Dict[str, Any],
+                     site: Optional[DataSite] = None) -> 'WbQuantity':
         """
         Create a WbQuantity from the JSON data given by the Wikibase API.
 
-        :param wb: Wikibase JSON
+        :param data: Wikibase JSON
         :param site: The Wikibase site
-        :rtype: pywikibot.WbQuantity
         """
-        amount = cls._todecimal(wb['amount'])
-        upperBound = cls._todecimal(wb.get('upperBound'))
-        lowerBound = cls._todecimal(wb.get('lowerBound'))
+        amount = cls._todecimal(data['amount'])
+        upperBound = cls._todecimal(data.get('upperBound'))
+        lowerBound = cls._todecimal(data.get('lowerBound'))
         bounds_provided = (upperBound is not None and lowerBound is not None)
         error = None
         if bounds_provided or cls._require_errors(site):
             error = (upperBound - amount, amount - lowerBound)
-        if wb['unit'] == '1':
+        if data['unit'] == '1':
             unit = None
         else:
-            unit = wb['unit']
+            unit = data['unit']
         return cls(amount, unit, error, site)
 
 
@@ -804,7 +798,7 @@ class WbMonolingualText(_WbRepresentation):
 
     _items = ('text', 'language')
 
-    def __init__(self, text: str, language: str):
+    def __init__(self, text: str, language: str) -> None:
         """
         Create a new WbMonolingualText object.
 
@@ -816,7 +810,7 @@ class WbMonolingualText(_WbRepresentation):
         self.text = text
         self.language = language
 
-    def toWikibase(self) -> dict:
+    def toWikibase(self) -> Dict[str, Any]:
         """
         Convert the data to a JSON object for the Wikibase API.
 
@@ -828,14 +822,15 @@ class WbMonolingualText(_WbRepresentation):
         return json
 
     @classmethod
-    def fromWikibase(cls, wb: dict):
+    def fromWikibase(cls: Type['WbMonolingualText'], data: Dict[str, Any],
+                     site: Optional[DataSite] = None) -> 'WbMonolingualText':
         """
         Create a WbMonolingualText from the JSON data given by Wikibase API.
 
-        :param wb: Wikibase JSON
-        :rtype: pywikibot.WbMonolingualText
+        :param data: Wikibase JSON
+        :param site: The Wikibase site
         """
-        return cls(wb['text'], wb['language'])
+        return cls(data['text'], data['language'])
 
 
 class _WbDataPage(_WbRepresentation):
@@ -850,7 +845,8 @@ class _WbDataPage(_WbRepresentation):
     _items = ('page', )
 
     @classmethod
-    def _get_data_site(cls, repo_site: DataSite) -> APISite:
+    def _get_data_site(cls: Type['_WbDataPage'], repo_site: DataSite
+                       ) -> APISite:
         """
         Return the site serving as a repository for a given data type.
 
@@ -861,7 +857,8 @@ class _WbDataPage(_WbRepresentation):
         raise NotImplementedError
 
     @classmethod
-    def _get_type_specifics(cls, site: DataSite) -> dict:
+    def _get_type_specifics(cls: Type['_WbDataPage'], site: DataSite
+                            ) -> Dict[str, Any]:
         """
         Return the specifics for a given data type.
 
@@ -879,15 +876,14 @@ class _WbDataPage(_WbRepresentation):
         raise NotImplementedError
 
     @staticmethod
-    def _validate(page, data_site, ending: str, label: str):
+    def _validate(page: 'Page', data_site: 'BaseSite', ending: str,
+                  label: str) -> None:
         """
         Validate the provided page against general and type specific rules.
 
         :param page: Page containing the data.
-        :type page: pywikibot.Page
         :param data_site: The site serving as a repository for the given
             data type.
-        :type data_site: APISite
         :param ending: Required filetype-like ending in page titles.
             E.g. '.map'
         :param label: Label describing the data type in error messages.
@@ -920,21 +916,20 @@ class _WbDataPage(_WbRepresentation):
                 "Page must be in 'Data:' namespace and end in '{}' "
                 'for {}.'.format(ending, label))
 
-    def __init__(self, page, site: Optional[DataSite] = None):
+    def __init__(self, page: 'Page', site: Optional[DataSite] = None) -> None:
         """
         Create a new _WbDataPage object.
 
         :param page: page containing the data
-        :type page: pywikibot.Page
         :param site: The Wikibase site
         """
-        site = site or Site().data_repository()
+        site = site or page.site.data_repository()
         specifics = type(self)._get_type_specifics(site)
         _WbDataPage._validate(page, specifics['data_site'],
                               specifics['ending'], specifics['label'])
         self.page = page
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         """Override super.hash() as toWikibase is a string for _WbDataPage."""
         return hash(self.toWikibase())
 
@@ -947,14 +942,18 @@ class _WbDataPage(_WbRepresentation):
         return self.page.title()
 
     @classmethod
-    def fromWikibase(cls, page_name: str, site: DataSite):
+    def fromWikibase(cls: Type['_WbDataPage'], page_name: str,
+                     site: Optional[DataSite]) -> '_WbDataPage':
         """
         Create a _WbDataPage from the JSON data given by the Wikibase API.
 
         :param page_name: page name from Wikibase value
         :param site: The Wikibase site
-        :rtype: pywikibot._WbDataPage
         """
+        # TODO: This method signature does not match our parent class (which
+        # takes a dictionary argument rather than a string). We should either
+        # change this method's signature or rename this method.
+
         data_site = cls._get_data_site(site)
         page = Page(data_site, page_name)
         return cls(page, site)
@@ -964,7 +963,7 @@ class WbGeoShape(_WbDataPage):
     """A Wikibase geo-shape representation."""
 
     @classmethod
-    def _get_data_site(cls, site: DataSite) -> APISite:
+    def _get_data_site(cls: Type['WbGeoShape'], site: DataSite) -> APISite:
         """
         Return the site serving as a geo-shape repository.
 
@@ -973,7 +972,8 @@ class WbGeoShape(_WbDataPage):
         return site.geo_shape_repository()
 
     @classmethod
-    def _get_type_specifics(cls, site: DataSite) -> dict:
+    def _get_type_specifics(cls: Type['WbGeoShape'], site: DataSite
+                            ) -> Dict[str, Any]:
         """
         Return the specifics for WbGeoShape.
 
@@ -991,7 +991,7 @@ class WbTabularData(_WbDataPage):
     """A Wikibase tabular-data representation."""
 
     @classmethod
-    def _get_data_site(cls, site: DataSite) -> APISite:
+    def _get_data_site(cls: Type['WbTabularData'], site: DataSite) -> APISite:
         """
         Return the site serving as a tabular-data repository.
 
@@ -1000,7 +1000,8 @@ class WbTabularData(_WbDataPage):
         return site.tabular_data_repository()
 
     @classmethod
-    def _get_type_specifics(cls, site: DataSite) -> dict:
+    def _get_type_specifics(cls: Type['WbTabularData'], site: DataSite
+                            ) -> Dict[str, Any]:
         """
         Return the specifics for WbTabularData.
 
@@ -1023,12 +1024,12 @@ class WbUnknown(_WbRepresentation):
 
     This data type is just a json container
 
-    *New in version 3.0.*
+    .. versionadded:: 3.0
     """
 
     _items = ('json',)
 
-    def __init__(self, json):
+    def __init__(self, json: Dict[str, Any]) -> None:
         """
         Create a new WbUnknown object.
 
@@ -1036,7 +1037,7 @@ class WbUnknown(_WbRepresentation):
         """
         self.json = json
 
-    def toWikibase(self) -> dict:
+    def toWikibase(self) -> Dict[str, Any]:
         """
         Return the JSON object for the Wikibase API.
 
@@ -1045,21 +1046,23 @@ class WbUnknown(_WbRepresentation):
         return self.json
 
     @classmethod
-    def fromWikibase(cls, json: dict):
+    def fromWikibase(cls: Type['WbUnknown'], data: Dict[str, Any],
+                     site: Optional[DataSite] = None) -> 'WbUnknown':
         """
         Create a WbUnknown from the JSON data given by the Wikibase API.
 
-        :param json: Wikibase JSON
-        :rtype: pywikibot.WbUnknown
+        :param data: Wikibase JSON
+        :param site: The Wikibase site
         """
-        return cls(json)
+        return cls(data)
 
 
-_sites = {}
+_sites = {}  # type: Dict[str, APISite]
 
 
 @cache
-def _code_fam_from_url(url: str, name: Optional[str] = None):
+def _code_fam_from_url(url: str, name: Optional[str] = None
+                       ) -> Tuple[str, str]:
     """Set url to cache and get code and family from cache.
 
     Site helper method.
@@ -1088,10 +1091,11 @@ def _code_fam_from_url(url: str, name: Optional[str] = None):
     return matched_sites[0]
 
 
-@_deprecate_arg('sysop', True)
-def Site(code: Optional[str] = None, fam=None, user: Optional[str] = None, *,
-         interface=None,
-         url: Optional[str] = None) -> Union[APISite, DataSite, ClosedSite]:
+def Site(code: Optional[str] = None,
+         fam: Union[str, 'Family', None] = None,
+         user: Optional[str] = None, *,
+         interface: Union[str, 'BaseSite', None] = None,
+         url: Optional[str] = None) -> BaseSite:
     """A factory method to obtain a Site object.
 
     Site objects are cached and reused by this method.
@@ -1139,18 +1143,16 @@ def Site(code: Optional[str] = None, fam=None, user: Optional[str] = None, *,
     :param code: language code (override config.mylang)
         code may also be a sitename like 'wikipedia:test'
     :param fam: family name or object (override config.family)
-    :type fam: str or pywikibot.family.Family
     :param user: bot user name to use on this site (override config.usernames)
     :param interface: site class or name of class in :py:obj:`pywikibot.site`
         (override config.site_interface)
-    :type interface: subclass of :py:obj:`pywikibot.site.BaseSite` or string
     :param url: Instead of code and fam, does try to get a Site based on the
         URL. Still requires that the family supporting that URL exists.
     :raises ValueError: URL and pair of code and family given
     :raises ValueError: Invalid interface name
+    :raises ValueError: Missing Site code
+    :raises ValueError: Missing Site family
     """
-    _logger = 'wiki'
-
     if url:
         # Either code and fam or url with optional fam for AutoFamily name
         if code:
@@ -1169,6 +1171,10 @@ def Site(code: Optional[str] = None, fam=None, user: Optional[str] = None, *,
         code = code or _config.mylang
         fam = fam or _config.family
 
+    if not (code and fam):
+        raise ValueError('Missing Site {}'
+                         .format('code' if not code else 'family'))
+
     if not isinstance(fam, Family):
         fam = Family.load(fam)
 
@@ -1200,7 +1206,7 @@ def Site(code: Optional[str] = None, fam=None, user: Optional[str] = None, *,
     if key not in _sites or not isinstance(_sites[key], interface):
         _sites[key] = interface(code=code, fam=fam, user=user)
         debug("Instantiated {} object '{}'"
-              .format(interface.__name__, _sites[key]), _logger)
+              .format(interface.__name__, _sites[key]))
 
         if _sites[key].code != code:
             warn('Site {} instantiated using different code "{}"'
@@ -1215,7 +1221,11 @@ from pywikibot.page import (  # noqa: E402
     Claim,
     FilePage,
     ItemPage,
+    LexemeForm,
+    LexemePage,
+    LexemeSense,
     Link,
+    MediaInfo,
     Page,
     PropertyPage,
     SiteLink,
@@ -1228,7 +1238,7 @@ from pywikibot.page import (  # noqa: E402
 link_regex = re.compile(r'\[\[(?P<title>[^\]|[<>{}]*)(\|.*?)?\]\]')
 
 
-def showDiff(oldtext, newtext, context=0):
+def showDiff(oldtext: str, newtext: str, context: int = 0) -> None:
     """
     Output a string showing the differences between oldtext and newtext.
 
@@ -1241,7 +1251,7 @@ def showDiff(oldtext, newtext, context=0):
 # Throttle and thread handling
 
 
-def sleep(secs):
+def sleep(secs: int) -> None:
     """Suspend execution of the current thread for the given number of seconds.
 
     Drop this process from the throttle log if wait time is greater than
@@ -1252,7 +1262,7 @@ def sleep(secs):
     time.sleep(secs)
 
 
-def stopme():
+def stopme() -> None:
     """
     Drop this process from the throttle log, after pending threads finish.
 
@@ -1264,18 +1274,16 @@ def stopme():
     _flush(False)
 
 
-def _flush(stop=True):
+def _flush(stop: bool = True) -> None:
     """
     Drop this process from the throttle log, after pending threads finish.
 
     Wait for the page-putter to flush its queue. Also drop this process from
     the throttle log. Called automatically at Python exit.
     """
-    _logger = 'wiki'
+    debug('_flush() called')
 
-    debug('_flush() called', _logger)
-
-    def remaining():
+    def remaining() -> Tuple[int, datetime.timedelta]:
         remainingPages = page_put_queue.qsize()
         if stop:
             # -1 because we added a None element to stop the queue
@@ -1291,9 +1299,9 @@ def _flush(stop=True):
 
     num, sec = remaining()
     if num > 0 and sec.total_seconds() > _config.noisysleep:
-        output(color_format(
-            '{lightblue}Waiting for {num} pages to be put. '
-            'Estimated time remaining: {sec}{default}', num=num, sec=sec))
+        output('<<lightblue>>Waiting for {num} pages to be put. '
+               'Estimated time remaining: {sec}<<default>>'
+               .format(num=num, sec=sec))
 
     if _putthread is not threading.current_thread():
         while (_putthread.is_alive()
@@ -1323,7 +1331,7 @@ atexit.register(_flush)
 
 
 # Create a separate thread for asynchronous page saves (and other requests)
-def async_manager():
+def async_manager() -> None:
     """Daemon; take requests from the queue and execute them in background."""
     while True:
         (request, args, kwargs) = page_put_queue.get()
@@ -1335,7 +1343,7 @@ def async_manager():
         page_put_queue_busy.get()
 
 
-def async_request(request, *args, **kwargs):
+def async_request(request: Callable, *args: Any, **kwargs: Any) -> None:
     """Put a request on the queue, and start the daemon if necessary."""
     if not _putthread.is_alive():
         with page_put_queue.mutex, suppress(AssertionError, RuntimeError):
@@ -1344,49 +1352,10 @@ def async_request(request, *args, **kwargs):
 
 
 # queue to hold pending requests
-page_put_queue = Queue(_config.max_queue_size)
+page_put_queue = Queue(_config.max_queue_size)  # type: Queue
 # queue to signal that async_manager is working on a request. See T147178.
-page_put_queue_busy = Queue(_config.max_queue_size)
+page_put_queue_busy = Queue(_config.max_queue_size)  # type: Queue
 # set up the background thread
-_putthread = threading.Thread(target=async_manager)
-# identification for debugging purposes
-_putthread.setName('Put-Thread')
-_putthread.setDaemon(True)
-
-wrapper = _ModuleDeprecationWrapper(__name__)
-wrapper.add_deprecated_attr('config2', replacement_name='pywikibot.config',
-                            since='20210426')
-wrapper.add_deprecated_attr('__release__', __version__,
-                            replacement_name='pywikibot.__version__',
-                            since='20200707')
-wrapper.add_deprecated_attr('showHelp', show_help,
-                            since='20200705')
-wrapper.add_deprecated_attr(
-    'unicode2html', replacement_name='pywikibot.tools.chars.string2html',
-    since='6.2.0')
-
-# This module aliases many (but not all) pywikibot.exception classes and one
-# from pywikibot.data.api. Use of these aliases is deprecated. When removed
-# we can drop them from both our import and __all__ listing.
-
-EXCEPTION_CLASSES = {
-    n for n, _ in inspect.getmembers(exceptions, inspect.isclass)
-}
-
-EXCEPTION_CLASSES.add('UploadWarning')
-EXCEPTION_CLASSES.update(DEPRECATED_EXCEPTIONS.keys())
-
-for name in __all__:
-    if name in EXCEPTION_CLASSES:
-        if name in DEPRECATED_EXCEPTIONS:
-            replacement = DEPRECATED_EXCEPTIONS[name]
-        elif name == 'UploadWarning':
-            replacement = 'UploadError'
-        else:
-            replacement = name
-
-        wrapper.add_deprecated_attr(
-            name,
-            replacement_name='pywikibot.exceptions.{}'.format(replacement),
-            since='20210424'
-        )
+_putthread = threading.Thread(target=async_manager,
+                              name='Put-Thread',  # for debugging purposes
+                              daemon=True)
