@@ -1,22 +1,17 @@
-"""Choices for input_choice."""
+"""Options and Choices for :py:meth:`pywikibot.input_choice`."""
 #
-# (C) Pywikibot team, 2015-2021
+# (C) Pywikibot team, 2015-2022
 #
 # Distributed under the terms of the MIT license.
 #
 import re
-
 from abc import ABC, abstractmethod
 from textwrap import fill
-from typing import Optional
+from typing import Any, Optional
 
 import pywikibot
-
-from pywikibot.tools import (
-    deprecated,
-    deprecated_args,
-    issue_deprecation_warning,
-)
+from pywikibot.backports import Iterable, Sequence
+from pywikibot.tools import deprecated, issue_deprecation_warning
 
 
 class Option(ABC):
@@ -25,9 +20,10 @@ class Option(ABC):
     A basic option for input_choice.
 
     The following methods need to be implemented:
-    * format(default=None)
-    * result(value)
-    * test(value)
+
+    - format(default=None)
+    - result(value)
+    - test(value)
 
     The methods ``test`` and ``handled`` are in such a relationship that
     when ``handled`` returns itself that ``test`` must return True for
@@ -38,18 +34,24 @@ class Option(ABC):
     True for the same value.
     """
 
-    def __init__(self, stop=True) -> None:
+    def __init__(self, stop: bool = True) -> None:
         """Initializer."""
         self._stop = stop
 
     @staticmethod
-    def formatted(text: str, options, default=None) -> str:
+    def formatted(text: str, options: Iterable['Option'],
+                  default: Optional[str] = None) -> str:
         """
         Create a text with the options formatted into it.
 
+        This static method is used by :py:meth:`pywikibot.input_choice`.
+        It calls :py:obj:`format` for all *options* to combine the
+        question for :py:meth:`pywikibot.input`.
+
         :param text: Text into which options are to be formatted
         :param options: Option instances to be formatted
-        :type options: Iterable
+        :param default: filler for any option's 'default' placeholder
+
         :return: Text with the options formatted into it
         """
         formatted_options = []
@@ -66,30 +68,29 @@ class Option(ABC):
         """Return whether this option stops asking."""
         return self._stop
 
-    def handled(self, value):
+    def handled(self, value: str) -> Optional['Option']:
         """
         Return the Option object that applies to the given value.
 
         If this Option object doesn't know which applies it returns None.
         """
-        if self.test(value):
-            return self
-        return None
+        return self if self.test(value) else None
 
-    def format(self, default=None):
+    def format(self, default: Optional[str] = None) -> str:
         """Return a formatted string for that option."""
         raise NotImplementedError()
 
-    def test(self, value):
+    def test(self, value: str) -> bool:
         """Return True whether this option applies."""
         raise NotImplementedError()
 
     @abstractmethod
-    def result(self, value):
+    def result(self, value: str) -> Any:
         """Return the actual value which is associated by the given one.
 
-        *New in version 6.2:* *result()* is an abstract method and must
-        be defined in subclasses
+        .. versionadded:: 6.2
+           *result()* is an abstract method and must be defined in
+           subclasses
         """
         raise NotImplementedError()
 
@@ -98,35 +99,46 @@ class OutputOption(Option):
 
     """An option that never stops and can output on each question.
 
-    :Note: OutputOption must have a an "out" property which returns a
-        string for output method.
+    :py:meth:`pywikibot.input_choice` uses before_question attribute to
+    decide whether to output before or after the question.
+
+    .. note:: OutputOption must have an :py:obj:`out` property which
+       returns a string for
+       :py:meth:`userinterface output()
+       <pywikibot.userinterfaces._interface_base.ABUIC.output>`
+       method.
     """
 
-    before_question = False
+    #: Place output before or after the question
+    before_question = False  # type: bool
 
     @property
-    def stop(self):
+    def stop(self) -> bool:
         """Never stop asking."""
         return False
 
-    def result(self, value):
+    def result(self, value: str) -> Any:
         """Just return None."""
         return None
 
     @property
     def out(self) -> str:
-        """String to be used when selected and possibly before the question.
+        """String to be used when selected before or after the question.
 
         :Note: This method is used by ui.input_choice instead of output().
 
-        *New in version 6.2.*
+        .. versionadded:: 6.2
         """
         return ''
 
-    def output(self):
-        """Output string when selected and possibly before the question.
+    @deprecated('pywikibot.output(OutputOption.out)', since='6.5')
+    def output(self) -> None:
+        """Output string.
 
-        :Note: This method should never be overridden.
+        .. deprecated:: 6.5
+           This method was replaced by :py:obj:`out` property and is no
+           no longer used by the
+           :py:mod:`userinterfaces <pywikibot.userinterfaces>` system.
         """
         pywikibot.output(self.out)
 
@@ -135,7 +147,7 @@ class StandardOption(Option):
 
     """An option with a description and shortcut and returning the shortcut."""
 
-    def __init__(self, option: str, shortcut: str, **kwargs):
+    def __init__(self, option: str, shortcut: str, **kwargs: Any) -> None:
         """
         Initializer.
 
@@ -146,7 +158,7 @@ class StandardOption(Option):
         self.option = option
         self.shortcut = shortcut.lower()
 
-    def format(self, default=None) -> str:
+    def format(self, default: Optional[str] = None) -> str:
         """Return a formatted string for that option."""
         index = self.option.lower().find(self.shortcut)
         shortcut = self.shortcut
@@ -158,11 +170,11 @@ class StandardOption(Option):
                 self.option[index + len(self.shortcut):])
         return '{} [{}]'.format(self.option, shortcut)
 
-    def result(self, value):
+    def result(self, value: str) -> Any:
         """Return the lowercased shortcut."""
         return self.shortcut
 
-    def test(self, value) -> bool:
+    def test(self, value: str) -> bool:
         """Return True whether this option applies."""
         return (self.shortcut.lower() == value.lower()
                 or self.option.lower() == value.lower())
@@ -172,19 +184,21 @@ class OutputProxyOption(OutputOption, StandardOption):
 
     """An option which calls out property of the given output class."""
 
-    def __init__(self, option, shortcut, output, **kwargs):
+    def __init__(self, option: str, shortcut: str, output: OutputOption,
+                 **kwargs: Any) -> None:
         """Create a new option for the given sequence."""
         super().__init__(option, shortcut, **kwargs)
         self._outputter = output
 
     @property
     def out(self) -> str:
-        """Return te contents."""
+        """Return the contents."""
         if not hasattr(self._outputter, 'out'):
             issue_deprecation_warning('{} without "out" property'
                                       .format(self.__class__.__name__),
                                       since='6.2.0')
-            return self._outputter.output()
+            self._outputter.output()
+            return ''
         return self._outputter.out
 
 
@@ -197,18 +211,19 @@ class NestedOption(OutputOption, StandardOption):
     option applies while handle returns the sub option.
     """
 
-    def __init__(self, option, shortcut, description, options):
+    def __init__(self, option: str, shortcut: str, description: str,
+                 options: Iterable[Option]) -> None:
         """Initializer."""
         super().__init__(option, shortcut, stop=False)
         self.description = description
         self.options = options
 
-    def format(self, default=None):
+    def format(self, default: Optional[str] = None) -> str:
         """Return a formatted string for that option."""
         self._output = Option.formatted(self.description, self.options)
         return super().format(default=default)
 
-    def handled(self, value):
+    def handled(self, value: str) -> Optional[Option]:
         """Return itself if it applies or the applying sub option."""
         for option in self.options:
             handled = option.handled(value)
@@ -218,7 +233,7 @@ class NestedOption(OutputOption, StandardOption):
         return super().handled(value)
 
     @property
-    def out(self):
+    def out(self) -> str:
         """Output of suboptions."""
         return self._output
 
@@ -227,9 +242,8 @@ class ContextOption(OutputOption, StandardOption):
 
     """An option to show more and more context."""
 
-    def __init__(
-        self, option, shortcut, text, context, delta=100, start=0, end=0
-    ):
+    def __init__(self, option: str, shortcut: str, text: str, context: int,
+                 delta: int = 100, start: int = 0, end: int = 0) -> None:
         """Initializer."""
         super().__init__(option, shortcut, stop=False)
         self.text = text
@@ -238,21 +252,20 @@ class ContextOption(OutputOption, StandardOption):
         self.start = start
         self.end = end
 
-    def result(self, value):
-        """Add the delta to the context and output it."""
+    def result(self, value: str) -> Any:
+        """Add the delta to the context."""
         self.context += self.delta
-        super().result(value)
+        return None
 
     @property
-    def out(self):
+    def out(self) -> str:
         """Output section of the text."""
         start = max(0, self.start - self.context)
         end = min(len(self.text), self.end + self.context)
         return self.text[start:end]
 
-    @deprecated_args(start_context='start', end_context='end')
     @deprecated('pywikibot.output(ContextOption.out)', since='6.2.0')
-    def output_range(self, start, end):
+    def output_range(self, start: int, end: int) -> None:
         """DEPRECATED. Output a section from the text."""
         pywikibot.output(self.text[start:end])
 
@@ -261,22 +274,27 @@ class Choice(StandardOption):
 
     """A simple choice consisting of an option, shortcut and handler."""
 
-    def __init__(self, option, shortcut, replacer):
+    def __init__(
+        self,
+        option: str,
+        shortcut: str,
+        replacer: Optional['pywikibot.bot.InteractiveReplace']
+    ) -> None:
         """Initializer."""
         super().__init__(option, shortcut)
         self._replacer = replacer
 
     @property
-    def replacer(self):
+    def replacer(self) -> Optional['pywikibot.bot.InteractiveReplace']:
         """The replacer."""
         return self._replacer
 
     @abstractmethod
-    def handle(self):
+    def handle(self) -> Any:
         """Handle this choice. Must be implemented."""
         raise NotImplementedError()
 
-    def handle_link(self):
+    def handle_link(self) -> bool:
         """The current link will be handled by this choice."""
         return False
 
@@ -285,12 +303,12 @@ class StaticChoice(Choice):
 
     """A static choice which just returns the given value."""
 
-    def __init__(self, option, shortcut, result):
+    def __init__(self, option: str, shortcut: str, result: Any) -> None:
         """Create instance with replacer set to None."""
         super().__init__(option, shortcut, None)
         self._result = result
 
-    def handle(self):
+    def handle(self) -> Any:
         """Return the predefined value."""
         return self._result
 
@@ -299,15 +317,24 @@ class LinkChoice(Choice):
 
     """A choice returning a mix of the link new and current link."""
 
-    def __init__(self, option, shortcut, replacer, replace_section,
-                 replace_label):
+    def __init__(
+        self,
+        option: str,
+        shortcut: str,
+        replacer: Optional['pywikibot.bot.InteractiveReplace'],
+        replace_section: bool,
+        replace_label: bool
+    ) -> None:
         """Initializer."""
         super().__init__(option, shortcut, replacer)
         self._section = replace_section
         self._label = replace_label
 
-    def handle(self):
+    def handle(self) -> Any:
         """Handle by either applying the new section or label."""
+        if not self.replacer:
+            raise ValueError('LinkChoice requires a replacer')
+
         kwargs = {}
         if self._section:
             kwargs['section'] = self.replacer._new.section
@@ -337,23 +364,27 @@ class AlwaysChoice(Choice):
 
     """Add an option to always apply the default."""
 
-    def __init__(self, replacer, option='always', shortcut='a'):
+    def __init__(self, replacer: Optional['pywikibot.bot.InteractiveReplace'],
+                 option: str = 'always', shortcut: str = 'a') -> None:
         """Initializer."""
         super().__init__(option, shortcut, replacer)
         self.always = False
 
-    def handle(self):
+    def handle(self) -> Any:
         """Handle the custom shortcut."""
         self.always = True
         return self.answer
 
-    def handle_link(self):
+    def handle_link(self) -> bool:
         """Directly return answer whether it's applying it always."""
         return self.always
 
     @property
-    def answer(self):
+    def answer(self) -> Any:
         """Get the actual default answer instructing the replacement."""
+        if not self.replacer:
+            raise ValueError('AlwaysChoice requires a replacer')
+
         return self.replacer.handle_answer(self.replacer._default)
 
 
@@ -361,7 +392,8 @@ class IntegerOption(Option):
 
     """An option allowing a range of integers."""
 
-    def __init__(self, minimum=1, maximum=None, prefix='', **kwargs):
+    def __init__(self, minimum: int = 1, maximum: Optional[int] = None,
+                 prefix: str = '', **kwargs: Any) -> None:
         """Initializer."""
         super().__init__(**kwargs)
         if not ((minimum is None or isinstance(minimum, int))
@@ -374,34 +406,36 @@ class IntegerOption(Option):
         self._max = maximum
         self.prefix = prefix
 
-    def test(self, value) -> bool:
+    def test(self, value: str) -> bool:
         """Return whether the value is an int and in the specified range."""
         try:
-            value = self.parse(value)
+            int_value = self.parse(value)
         except ValueError:
             return False
 
-        return ((self.minimum is None or value >= self.minimum)
-                and (self.maximum is None or value <= self.maximum))
+        return ((self.minimum is None or int_value >= self.minimum)
+                and (self.maximum is None or int_value <= self.maximum))
 
     @property
-    def minimum(self):
+    def minimum(self) -> int:
         """Return the lower bound of the range of allowed values."""
         return self._min
 
     @property
-    def maximum(self):
+    def maximum(self) -> Optional[int]:
         """Return the upper bound of the range of allowed values."""
         return self._max
 
-    def format(self, default=None) -> str:
+    def format(self, default: Optional[str] = None) -> str:
         """Return a formatted string showing the range."""
+        value = None  # type: Optional[int]
+
         if default is not None and self.test(default):
             value = self.parse(default)
             default = '[{}]'.format(value)
         else:
-            value = None
             default = ''
+
         if self.minimum is not None or self.maximum is not None:
             if default and value == self.minimum:
                 minimum = default
@@ -422,14 +456,14 @@ class IntegerOption(Option):
             rng = 'any' + default
         return '{}<number> [{}]'.format(self.prefix, rng)
 
-    def parse(self, value) -> int:
+    def parse(self, value: str) -> int:
         """Return integer from value with prefix removed."""
         if value.lower().startswith(self.prefix.lower()):
             return int(value[len(self.prefix):])
         raise ValueError('Value does not start with prefix')
 
-    def result(self, value):
-        """Return the value converted into int."""
+    def result(self, value: str) -> Any:
+        """Return a tuple with the prefix and value converted into an int."""
         return self.prefix, self.parse(value)
 
 
@@ -437,7 +471,8 @@ class ListOption(IntegerOption):
 
     """An option to select something from a list."""
 
-    def __init__(self, sequence, prefix='', **kwargs):
+    def __init__(self, sequence: Sequence[str], prefix: str = '',
+                 **kwargs: Any) -> None:
         """Initializer."""
         self._list = sequence
         try:
@@ -446,7 +481,7 @@ class ListOption(IntegerOption):
             raise ValueError('The sequence is empty.')
         del self._max
 
-    def format(self, default=None):
+    def format(self, default: Optional[str] = None) -> str:
         """Return a string showing the range."""
         if not self._list:
             raise ValueError('The sequence is empty.')
@@ -458,7 +493,7 @@ class ListOption(IntegerOption):
         """Return the maximum value."""
         return len(self._list)
 
-    def result(self, value):
+    def result(self, value: str) -> Any:
         """Return a tuple with the prefix and selected value."""
         return self.prefix, self._list[self.parse(value) - 1]
 
@@ -467,13 +502,14 @@ class ShowingListOption(ListOption, OutputOption):
 
     """An option to show a list and select an item.
 
-    *New in version 3.0.*
+    .. versionadded:: 3.0
     """
 
     before_question = True
 
-    def __init__(self, sequence, prefix='', pre: Optional[str] = None,
-                 post: Optional[str] = None, **kwargs):
+    def __init__(self, sequence: Sequence[str], prefix: str = '',
+                 pre: Optional[str] = None, post: Optional[str] = None,
+                 **kwargs: Any) -> None:
         """Initializer.
 
         :param pre: Additional comment printed before the list.
@@ -484,12 +520,12 @@ class ShowingListOption(ListOption, OutputOption):
         self.post = post
 
     @property
-    def stop(self):
+    def stop(self) -> bool:
         """Return whether this option stops asking."""
         return self._stop
 
     @property
-    def out(self):
+    def out(self) -> str:
         """Output text of the enumerated list."""
         text = ''
         if self.pre is not None:
@@ -506,10 +542,10 @@ class MultipleChoiceList(ListOption):
 
     """An option to select multiple items from a list.
 
-    *New in version 3.0.*
+    .. versionadded 3.0
     """
 
-    def test(self, value) -> bool:
+    def test(self, value: str) -> bool:
         """Return whether the values are int and in the specified range."""
         try:
             values = [self.parse(val) for val in value.split(',')]
@@ -526,7 +562,7 @@ class MultipleChoiceList(ListOption):
 
         return False
 
-    def result(self, value):
+    def result(self, value: str) -> Any:
         """Return a tuple with the prefix and selected values as a list."""
         values = (self.parse(val) for val in value.split(','))
         result = [self._list[val - 1] for val in values]
@@ -537,7 +573,7 @@ class ShowingMultipleChoiceList(ShowingListOption, MultipleChoiceList):
 
     """An option to show a list and select multiple items.
 
-    *New in version 3.0.*
+    .. versionadded 3.0
     """
 
 
@@ -548,52 +584,53 @@ class HighlightContextOption(ContextOption):
     color = 'lightred'
 
     @property
-    def out(self):
+    def out(self) -> str:
         """Highlighted output section of the text."""
         start = max(0, self.start - self.context)
         end = min(len(self.text), self.end + self.context)
-        color_format = pywikibot.tools.formatter.color_format
-        return color_format('{}{%(color)s}{}{default}{}'
-                            % {'color': self.color},
-                            self.text[start:self.start],
-                            self.text[self.start:self.end],
-                            self.text[self.end:end])
+        return '{}<<{color}>>{}<<default>>{}'.format(
+            self.text[start:self.start],
+            self.text[self.start:self.end],
+            self.text[self.end:end],
+            color=self.color)
 
-    @deprecated('pywikibot.output(HighlightContextOption.out)',
-                since='6.2.0')
-    def output_range(self, start, end):
-        """DEPRECATED. Show normal context with a highlighted center region."""
-        color_format = pywikibot.tools.formatter.color_format
-        text = color_format('{}{%(color)s}{}{default}{}'
-                            % {'color': self.color},
-                            self.text[start:self.start],
-                            self.text[self.start:self.end],
-                            self.text[self.end:end])
+    @deprecated('pywikibot.output(HighlightContextOption.out)', since='6.2.0')
+    def output_range(self, start: int, end: int) -> None:
+        """Show normal context with a highlighted center region.
+
+        .. deprecated:: 6.2
+           use :attr:`out` instead.
+        """
+        text = '{}<<{color}>>{}<<default>>{}'.format(
+            self.text[start:self.start],
+            self.text[self.start:self.end],
+            self.text[self.end:end],
+            color=self.color)
         pywikibot.output(text)
 
 
-class UnhandledAnswer(Exception):
+class UnhandledAnswer(Exception):  # noqa: N818
 
     """The given answer didn't suffice."""
 
-    def __int__(self, stop=False):
+    def __int__(self, stop: bool = False) -> None:
         """Initializer."""
         self.stop = stop
 
 
-class ChoiceException(StandardOption, Exception):
+class ChoiceException(StandardOption, Exception):  # noqa: N818
 
     """A choice for input_choice which result in this exception."""
 
-    def result(self, value):
+    def result(self, value: Any) -> Any:
         """Return itself to raise the exception."""
         return self
 
 
-class QuitKeyboardInterrupt(ChoiceException, KeyboardInterrupt):
+class QuitKeyboardInterrupt(ChoiceException, KeyboardInterrupt):  # noqa: N818
 
     """The user has cancelled processing at a prompt."""
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Constructor using the 'quit' ('q') in input_choice."""
         super().__init__('quit', 'q')
