@@ -7,15 +7,14 @@ This module requires sseclient to be installed::
 
     pip install sseclient
 
-*New in version 3.0.*
+.. versionadded:: 3.0
 """
 #
-# (C) Pywikibot team, 2017-2021
+# (C) Pywikibot team, 2017-2022
 #
 # Distributed under the terms of the MIT license.
 #
 import json
-import socket
 from functools import partial
 from typing import Optional
 
@@ -25,7 +24,6 @@ from requests.packages.urllib3.exceptions import ProtocolError
 from requests.packages.urllib3.util.response import httplib
 
 from pywikibot import Site, Timestamp, config, debug, warning
-from pywikibot.tools import deprecated_args
 
 
 try:
@@ -40,9 +38,6 @@ if parse_version(requests_version) < parse_version('2.20.1'):
         "install it with 'pip install \"requests>=2.20.1\"'\n")
 
 
-_logger = 'pywikibot.eventstreams'
-
-
 class EventStreams:
 
     """Basic EventStreams iterator class for Server-Sent Events (SSE) protocol.
@@ -76,8 +71,7 @@ class EventStreams:
     >>> del stream
     """
 
-    @deprecated_args(stream='streams')
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs) -> None:
         """Initializer.
 
         :keyword site: a project site object. Used when no url is given
@@ -127,7 +121,7 @@ class EventStreams:
         kwargs.setdefault('timeout', config.socket_timeout)
         self.sse_kwargs = kwargs
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """Return representation string."""
         kwargs = self.sse_kwargs.copy()
         if self._site != Site():
@@ -161,7 +155,7 @@ class EventStreams:
                                         if self._since else '')))
         return self._url
 
-    def set_maximum_items(self, value: int):
+    def set_maximum_items(self, value: int) -> None:
         """
         Set the maximum number of items to be retrieved from the stream.
 
@@ -174,7 +168,7 @@ class EventStreams:
         if value is not None:
             self._total = int(value)
             debug('{}: Set limit (maximum_items) to {}.'
-                  .format(self.__class__.__name__, self._total), _logger)
+                  .format(self.__class__.__name__, self._total))
 
     def register_filter(self, *args, **kwargs):
         """Register a filter.
@@ -297,7 +291,7 @@ class EventStreams:
                         "Install it with 'pip install \"sseclient>=0.0.18\"'")
             try:
                 event = next(self.source)
-            except (ProtocolError, socket.error, httplib.IncompleteRead) as e:
+            except (ProtocolError, OSError, httplib.IncompleteRead) as e:
                 warning('Connection error: {}.\n'
                         'Try to re-establish connection.'.format(e))
                 del self.source
@@ -325,7 +319,7 @@ class EventStreams:
                 warning('Unknown event {} occurred.'.format(event.event))
 
         debug('{}: Stopped iterating due to exceeding item limit.'
-              .format(self.__class__.__name__), _logger)
+              .format(self.__class__.__name__))
         del self.source
 
 
