@@ -1,4 +1,3 @@
-#!/usr/bin/python
 """
 This module can do slight modifications to tidy a wiki page's source code.
 
@@ -51,31 +50,25 @@ or by adding a list to the given one::
                                      'your_script_name_2']
 """
 #
-# (C) Pywikibot team, 2006-2021
+# (C) Pywikibot team, 2006-2022
 #
 # Distributed under the terms of the MIT license.
 #
 import re
-
 from enum import IntEnum
-from typing import Optional
+from typing import Any, Union
+from urllib.parse import urlparse, urlunparse
 
 import pywikibot
 from pywikibot import textlib
+from pywikibot.backports import Callable, Dict, List, Match, Pattern
 from pywikibot.exceptions import InvalidTitleError
 from pywikibot.textlib import (
     FILE_LINK_REGEX,
     MultiTemplateMatchBuilder,
     _get_regexes,
 )
-from pywikibot.tools import (
-    deprecated,
-    deprecated_args,
-    first_lower,
-    first_upper,
-    issue_deprecation_warning,
-    ModuleDeprecationWrapper,
-)
+from pywikibot.tools import first_lower, first_upper
 from pywikibot.tools.chars import url2string
 
 
@@ -184,7 +177,7 @@ class CANCEL(IntEnum):
     If an error occurred and either skips the page or the method
     or a single match. ALL raises the exception.
 
-    *New in version 6.3.*
+    .. versionadded:: 6.3
     """
 
     ALL = 0
@@ -193,7 +186,7 @@ class CANCEL(IntEnum):
     MATCH = 3
 
 
-def _format_isbn_match(match, strict=True):
+def _format_isbn_match(match: Match[str], strict: bool = True) -> str:
     """Helper function to validate and format a single matched ISBN."""
     if not stdnum_isbn:
         raise NotImplementedError(
@@ -211,7 +204,7 @@ def _format_isbn_match(match, strict=True):
     return stdnum_isbn.format(isbn)
 
 
-def _reformat_ISBNs(text, strict=True):
+def _reformat_ISBNs(text: str, strict: bool = True) -> str:
     """Helper function to normalise ISBNs in text.
 
     :raises Exception: Invalid ISBN encountered when strict enabled
@@ -222,44 +215,33 @@ def _reformat_ISBNs(text, strict=True):
 
 class CosmeticChangesToolkit:
 
-    """Cosmetic changes toolkit."""
+    """Cosmetic changes toolkit.
 
-    @deprecated_args(redirect=True, diff='show_diff', site='page')
-    def __init__(self, page, *,
+    .. versionchanged:: 7.0
+       `from_page()` method was removed
+    """
+
+    def __init__(self, page: 'pywikibot.page.BasePage', *,
                  show_diff: bool = False,
-                 namespace: Optional[int] = None,
-                 pageTitle: Optional[str] = None,
-                 ignore: IntEnum = CANCEL.ALL):
+                 ignore: IntEnum = CANCEL.ALL) -> None:
         """Initializer.
 
+        .. versionchanged:: 5.2
+           instantiate the CosmeticChangesToolkit from a page object;
+           only allow keyword arguments except for page parameter;
+           `namespace` and `pageTitle` parameters are deprecated
+
+        .. versionchanged:: 7.0
+           `namespace` and `pageTitle` parameters were removed
+
         :param page: the Page object containing the text to be modified
-        :type page: pywikibot.Page
-        :param show_diff: show difference after replacements (default: False)
-        :param namespace: DEPRECATED namespace parameter
-        :param pageTitle: DEPRECATED page title parameter
+        :param show_diff: show difference after replacements
         :param ignore: ignores if an error occurred and either skips the page
             or only that method. It can be set one of the CANCEL constants
         """
-        if isinstance(page, pywikibot.BaseSite):
-            self.site = page
-            self.title = pageTitle
-            try:
-                self.namespace = self.site.namespaces.resolve(namespace).pop(0)
-            except (KeyError, TypeError, IndexError):
-                raise ValueError('{} needs a valid namespace'
-                                 .format(self.__class__.__name__))
-            issue_deprecation_warning(
-                'site parameter of CosmeticChangesToolkit',
-                'a pywikibot.Page object as first parameter',
-                since='20201102')
-        else:
-            if namespace is not None or pageTitle is not None:
-                raise TypeError(
-                    "'namespace' and 'pageTitle' arguments are invalid with "
-                    'a given Page object')
-            self.site = page.site
-            self.title = page.title()
-            self.namespace = page.namespace()
+        self.site = page.site
+        self.title = page.title()
+        self.namespace = page.namespace()
 
         self.show_diff = show_diff
         self.template = (self.namespace == 10)
@@ -292,27 +274,7 @@ class CosmeticChangesToolkit:
         if stdnum_isbn:
             self.common_methods.append(self.fix_ISBN)
 
-    @property
-    @deprecated('show_diff', since='20200415')
-    def diff(self):
-        """CosmeticChangesToolkit.diff attribute getter."""
-        return self.show_diff
-
-    @diff.setter
-    @deprecated('show_diff', since='20200415')
-    def diff(self, value):
-        """CosmeticChangesToolkit.diff attribute setter."""
-        self.show_diff = bool(value)
-
-    @classmethod
-    @deprecated('CosmeticChangesToolkit with pywikibot.Page object',
-                since='20200415')
-    @deprecated_args(diff='show_diff')
-    def from_page(cls, page, show_diff=False, ignore=CANCEL.ALL):
-        """Create toolkit based on the page."""
-        return cls(page, show_diff=show_diff, ignore=ignore)
-
-    def safe_execute(self, method, text):
+    def safe_execute(self, method: Callable[[str], str], text: str) -> str:
         """Execute the method and catch exceptions if enabled."""
         result = None
         try:
@@ -321,18 +283,18 @@ class CosmeticChangesToolkit:
             if self.ignore == CANCEL.METHOD:
                 pywikibot.warning('Unable to perform "{}" on "{}"!'
                                   .format(method.__name__, self.title))
-                pywikibot.exception(e)
+                pywikibot.error(e)
             else:
                 raise
         return text if result is None else result
 
-    def _change(self, text):
+    def _change(self, text: str) -> str:
         """Execute all clean up methods."""
         for method in self.common_methods:
             text = self.safe_execute(method, text)
         return text
 
-    def change(self, text):
+    def change(self, text: str) -> Union[bool, str]:
         """Execute all clean up methods and catch errors if activated."""
         try:
             new_text = self._change(text)
@@ -340,7 +302,7 @@ class CosmeticChangesToolkit:
             if self.ignore == CANCEL.PAGE:
                 pywikibot.warning('Skipped "{}", because an error occurred.'
                                   .format(self.title))
-                pywikibot.exception(e)
+                pywikibot.error(e)
                 return False
             raise
         else:
@@ -348,7 +310,7 @@ class CosmeticChangesToolkit:
                 pywikibot.showDiff(text, new_text)
             return new_text
 
-    def fixSelfInterwiki(self, text):
+    def fixSelfInterwiki(self, text: str) -> str:
         """
         Interwiki links to the site itself are displayed like local links.
 
@@ -360,7 +322,7 @@ class CosmeticChangesToolkit:
             text = interwikiR.sub(r'[[\1]]', text)
         return text
 
-    def standardizePageFooter(self, text):
+    def standardizePageFooter(self, text: str) -> str:
         """
         Standardize page footer.
 
@@ -375,8 +337,10 @@ class CosmeticChangesToolkit:
         2. additional information depending on the local site policy
         3. interwiki
         """
+        assert self.title is not None
+
         categories = []
-        interwiki_links = []
+        interwiki_links = {}
 
         # get categories
         if not self.template:
@@ -422,7 +386,7 @@ class CosmeticChangesToolkit:
 
         return text
 
-    def translateAndCapitalizeNamespaces(self, text):
+    def translateAndCapitalizeNamespaces(self, text: str) -> str:
         """Use localized namespace names."""
         # arz uses English stylish codes
         if self.site.sitename == 'wikipedia:arz':
@@ -482,7 +446,7 @@ class CosmeticChangesToolkit:
                         exceptions)
         return text
 
-    def translateMagicWords(self, text):
+    def translateMagicWords(self, text: str) -> str:
         """Use localized magic words."""
         # not wanted at ru
         # arz uses English stylish codes
@@ -490,7 +454,7 @@ class CosmeticChangesToolkit:
         if self.site.code in ['arz', 'en', 'ru']:
             return text
 
-        def init_cache():
+        def init_cache() -> None:
             for magicword in ('img_thumbnail', 'img_left', 'img_center',
                               'img_right', 'img_none', 'img_framed',
                               'img_frameless', 'img_border', 'img_upright',
@@ -504,7 +468,7 @@ class CosmeticChangesToolkit:
             if not cache:
                 cache[False] = True  # signal there is nothing to replace
 
-        def replace_magicword(match):
+        def replace_magicword(match: Match[str]) -> str:
             if cache.get(False):
                 return match.group()
             split = match.group().split('|')
@@ -519,7 +483,7 @@ class CosmeticChangesToolkit:
             return '{}|{}]]'.format(
                 split[0], '|'.join(cache.get(x.strip(), x) for x in split[1:]))
 
-        cache = {}
+        cache = {}  # type: Dict[Union[bool, str], Any]
         exceptions = ['comment', 'nowiki', 'pre', 'syntaxhighlight']
         regex = re.compile(
             FILE_LINK_REGEX % '|'.join(self.site.namespaces[6]),
@@ -548,17 +512,13 @@ class CosmeticChangesToolkit:
         """
         # helper function which works on one link and either returns it
         # unmodified, or returns a replacement.
-        def handleOneLink(match):
+        def handleOneLink(match: Match[str]) -> str:
             titleWithSection = match.group('titleWithSection')
             label = match.group('label')
             trailingChars = match.group('linktrail')
             newline = match.group('newline')
 
-            try:
-                is_interwiki = self.site.isInterwikiLink(titleWithSection)
-            except ValueError:  # T111513
-                is_interwiki = True
-
+            is_interwiki = self.site.isInterwikiLink(titleWithSection)
             if is_interwiki:
                 return match.group()
 
@@ -681,7 +641,7 @@ class CosmeticChangesToolkit:
                                       'startspace'])
         return text
 
-    def resolveHtmlEntities(self, text):
+    def resolveHtmlEntities(self, text: str) -> str:
         """Replace HTML entities with string."""
         ignore = [
             38,     # Ampersand (&amp;)
@@ -709,7 +669,7 @@ class CosmeticChangesToolkit:
             text, ignore=ignore, exceptions=['comment', 'syntaxhighlight'])
         return text
 
-    def removeEmptySections(self, text):
+    def removeEmptySections(self, text: str) -> str:
         """Cleanup empty sections."""
         # userspace contains article stubs without nobots/in use templates
         if self.namespace == 2:
@@ -753,7 +713,7 @@ class CosmeticChangesToolkit:
                 new_body.extend(sections[i])
         return header + ''.join(new_body) + footer
 
-    def removeUselessSpaces(self, text):
+    def removeUselessSpaces(self, text: str) -> str:
         """Cleanup multiple or trailing spaces."""
         exceptions = ['comment', 'math', 'nowiki', 'pre', 'syntaxhighlight',
                       'startspace', 'table']
@@ -763,7 +723,7 @@ class CosmeticChangesToolkit:
                                      exceptions, site=self.site)
         return text
 
-    def removeNonBreakingSpaceBeforePercent(self, text):
+    def removeNonBreakingSpaceBeforePercent(self, text: str) -> str:
         """
         Remove a non-breaking space between number and percent sign.
 
@@ -775,7 +735,7 @@ class CosmeticChangesToolkit:
             text, r'(\d)&(?:nbsp|#160|#x[Aa]0);%', r'\1 %', ['timeline'])
         return text
 
-    def cleanUpSectionHeaders(self, text):
+    def cleanUpSectionHeaders(self, text: str) -> str:
         """
         Add a space between the equal signs and the section title.
 
@@ -801,7 +761,7 @@ class CosmeticChangesToolkit:
             r'\1 \g<title> \1\n',
             ['comment', 'math', 'nowiki', 'pre'])
 
-    def putSpacesInLists(self, text):
+    def putSpacesInLists(self, text: str) -> str:
         """
         Add a space between the * or # and the text.
 
@@ -822,7 +782,7 @@ class CosmeticChangesToolkit:
                 exceptions)
         return text
 
-    def replaceDeprecatedTemplates(self, text):
+    def replaceDeprecatedTemplates(self, text: str) -> str:
         """Replace deprecated templates."""
         exceptions = ['comment', 'math', 'nowiki', 'pre']
         builder = MultiTemplateMatchBuilder(self.site)
@@ -845,9 +805,9 @@ class CosmeticChangesToolkit:
         return text
 
     # from fixes.py
-    def fixSyntaxSave(self, text):
+    def fixSyntaxSave(self, text: str) -> str:
         """Convert weblinks to wikilink, fix link syntax."""
-        def replace_link(match):
+        def replace_link(match: Match[str]) -> str:
             """Create a string to replace a single link."""
             replacement = '[['
             if re.match(r'(?:' + '|'.join(list(self.site.namespaces[6])
@@ -861,6 +821,7 @@ class CosmeticChangesToolkit:
 
         exceptions = ['comment', 'math', 'nowiki', 'pre', 'startspace',
                       'syntaxhighlight']
+
         # link to the wiki working on
         # Only use suffixes for article paths
         for suffix in self.site._interwiki_urls(True):
@@ -869,31 +830,43 @@ class CosmeticChangesToolkit:
                 https_url = None
             else:
                 https_url = self.site.base_url(suffix, 'https')
+
             # compare strings without the protocol, if they are empty support
             # also no prefix (//en.wikipedia.org/…)
-            if https_url is not None and http_url[4:] == https_url[5:]:
-                urls = ['(?:https?:)?' + re.escape(http_url[5:])]
+            http = urlparse(http_url)
+            https = urlparse(https_url)
+            if https_url is not None and http.netloc == https.netloc:
+                urls = ['(?:https?:)?'
+                        + re.escape(urlunparse(('', *http[1:])))]
             else:
                 urls = [re.escape(url) for url in (http_url, https_url)
                         if url is not None]
+
             for url in urls:
-                # Only include links which don't include the separator as
-                # the wikilink won't support additional parameters
-                separator = '?'
-                if '?' in suffix:
-                    separator += '&'
+                # unescape {} placeholder
+                url = url.replace(r'\{\}', '{title}')
+
+                # Only include links which don't include the separator
+                # as the wikilink won't support additional parameters
+                separator = '?&' if '?' in suffix else '?'
+
                 # Match first a non space in the title to prevent that multiple
                 # spaces at the end without title will be matched by it
+                title_regex = (r'(?P<link>[^{sep}]+?)'
+                               r'(\s+(?P<title>[^\s].*?))'
+                               .format(sep=separator))
+                url_regex = r'\[\[?{url}?\s*\]\]?'.format(url=url)
                 text = textlib.replaceExcept(
                     text,
-                    r'\[\[?' + url + r'(?P<link>[^' + separator + r']+?)'
-                    r'(\s+(?P<title>[^\s].*?))?\s*\]\]?',
+                    url_regex.format(title=title_regex),
                     replace_link, exceptions, site=self.site)
+
         # external link in/starting with double brackets
         text = textlib.replaceExcept(
             text,
             r'\[\[(?P<url>https?://[^\]]+?)\]\]?',
             r'[\g<url>]', exceptions, site=self.site)
+
         # external link and description separated by a pipe, with
         # whitespace in front of the pipe, so that it is clear that
         # the dash is not a legitimate part of the URL.
@@ -901,6 +874,7 @@ class CosmeticChangesToolkit:
             text,
             r'\[(?P<url>https?://[^\|\] \r\n]+?) +\| *(?P<label>[^\|\]]+?)\]',
             r'[\g<url> \g<label>]', exceptions)
+
         # dash in external link, where the correct end of the URL can
         # be detected from the file extension. It is very unlikely that
         # this will cause mistakes.
@@ -913,9 +887,9 @@ class CosmeticChangesToolkit:
             r'[\g<url> \g<label>]', exceptions)
         return text
 
-    def fixHtml(self, text):
+    def fixHtml(self, text: str) -> str:
         """Relace html markups with wikitext markups."""
-        def replace_header(match):
+        def replace_header(match: Match[str]) -> str:
             """Create a header string for replacing."""
             depth = int(match.group(1))
             return r'{0} {1} {0}'.format('=' * depth, match.group(2))
@@ -945,7 +919,7 @@ class CosmeticChangesToolkit:
         # TODO: maybe we can make the bot replace <p> tags with \r\n's.
         return text
 
-    def fixReferences(self, text):
+    def fixReferences(self, text: str) -> str:
         """Fix references tags."""
         # See also
         # https://en.wikipedia.org/wiki/User:AnomieBOT/source/tasks/OrphanReferenceFixer.pm
@@ -963,7 +937,7 @@ class CosmeticChangesToolkit:
                                      r'<ref \1/>', exceptions)
         return text
 
-    def fixStyle(self, text):
+    def fixStyle(self, text: str) -> str:
         """Convert prettytable to wikitable class."""
         exceptions = ['comment', 'math', 'nowiki', 'pre', 'startspace',
                       'syntaxhighlight']
@@ -973,10 +947,21 @@ class CosmeticChangesToolkit:
                                          r'\1wikitable\2', exceptions)
         return text
 
-    def fixTypo(self, text):
+    def fixTypo(self, text: str) -> str:
         """Fix units."""
-        exceptions = ['comment', 'gallery', 'hyperlink', 'interwiki', 'link',
-                      'nowiki', 'math', 'pre', 'startspace', 'syntaxhighlight']
+        exceptions = [
+            'comment',
+            'gallery',
+            'hyperlink',
+            'interwiki',
+            'link',
+            'nowiki',
+            'math',
+            'pre',
+            'startspace',
+            'syntaxhighlight',
+        ]  # type: List[Union[str, Pattern[str]]]
+
         # change <number> ccm -> <number> cm³
         text = textlib.replaceExcept(text, r'(\d)\s*(?:&nbsp;)?ccm',
                                      r'\1&nbsp;cm³', exceptions,
@@ -993,7 +978,7 @@ class CosmeticChangesToolkit:
                                      site=self.site)
         return text
 
-    def fixArabicLetters(self, text):
+    def fixArabicLetters(self, text: str) -> str:
         """Fix Arabic and Persian letters."""
         if self.site.code not in ['ckb', 'fa']:
             return text
@@ -1013,7 +998,7 @@ class CosmeticChangesToolkit:
             'ref',
             'startspace',
             'syntaxhighlight',
-        ]
+        ]  # type: List[Union[str, Pattern[str]]]
 
         digits = textlib.NON_LATIN_DIGITS
         faChrs = 'ءاآأإئؤبپتثجچحخدذرزژسشصضطظعغفقکگلمنوهیةيك' + digits['fa']
@@ -1039,7 +1024,7 @@ class CosmeticChangesToolkit:
 
         return text
 
-    def commonsfiledesc(self, text):
+    def commonsfiledesc(self, text: str) -> str:
         """
         Clean up file descriptions on Wikimedia Commons.
 
@@ -1105,26 +1090,6 @@ class CosmeticChangesToolkit:
             r'\1== {{int:license-header}} ==', exceptions, True)
         return text
 
-    def fix_ISBN(self, text):
+    def fix_ISBN(self, text: str) -> str:
         """Hyphenate ISBN numbers."""
         return _reformat_ISBNs(text, strict=self.ignore != CANCEL.MATCH)
-
-
-_CANCEL_ALL = CANCEL.ALL
-_CANCEL_PAGE = CANCEL.PAGE
-_CANCEL_METHOD = CANCEL.METHOD
-_CANCEL_MATCH = CANCEL.MATCH
-
-wrapper = ModuleDeprecationWrapper(__name__)
-wrapper.add_deprecated_attr('CANCEL_ALL', _CANCEL_ALL,
-                            replacement_name='CANCEL.ALL',
-                            since='20210528')
-wrapper.add_deprecated_attr('CANCEL_PAGE', _CANCEL_PAGE,
-                            replacement_name='CANCEL.PAGE',
-                            since='20210528')
-wrapper.add_deprecated_attr('CANCEL_METHOD', _CANCEL_METHOD,
-                            replacement_name='CANCEL.METHOD',
-                            since='20210528')
-wrapper.add_deprecated_attr('CANCEL_MATCH', _CANCEL_MATCH,
-                            replacement_name='CANCEL.MATCH',
-                            since='20210528')
