@@ -1,22 +1,33 @@
 """Date data and manipulation module."""
 #
-# (C) Pywikibot team, 2003-2021
+# (C) Pywikibot team, 2003-2022
 #
 # Distributed under the terms of the MIT license.
 #
 import calendar
 import datetime
 import re
-from collections import defaultdict
-from collections.abc import Mapping, MutableMapping
+from collections import abc, defaultdict
 from contextlib import suppress
 from functools import singledispatch
 from string import digits as _decimalDigits  # noqa: N812
+from typing import Optional, Union
 
+import pywikibot.site
 from pywikibot import Site
-from pywikibot.backports import Tuple
+from pywikibot.backports import (
+    Any,
+    Callable,
+    Dict,
+    Iterator,
+    List,
+    Mapping,
+    Pattern,
+    Sequence,
+    Tuple,
+)
 from pywikibot.textlib import NON_LATIN_DIGITS
-from pywikibot.tools import deprecated, first_lower, first_upper
+from pywikibot.tools import first_lower, first_upper
 
 
 #
@@ -42,10 +53,22 @@ centuryFormats = ['CenturyAD', 'CenturyBC']
 yearFormats = ['YearAD', 'YearBC']
 millFormats = ['MillenniumAD', 'MillenniumBC']
 snglValsFormats = ['CurrEvents']
+tuplst_type = List[Tuple[Callable[[Union[int, str]], Any],
+                         Callable[[Union[int, str]], bool]]]
+encf_type = Callable[[int], Union[int, Sequence[int]]]
+decf_type = Callable[[Sequence[int]], int]
+
+# decoders are three value tuples, with an optional fourth to represent a
+# required number of digits
+
+decoder_type = Union[
+    Tuple[str, Callable[[int], str], Callable[[str], int]],
+    Tuple[str, Callable[[int], str], Callable[[str], int], int]
+]
 
 
 @singledispatch
-def multi(value: int, tuplst):
+def multi(value: int, tuplst: tuplst_type) -> Any:
     """
     Run multiple pattern checks for the same entry.
 
@@ -65,7 +88,7 @@ def multi(value: int, tuplst):
 
 
 @multi.register(str)
-def _(value: str, tuplst):
+def _(value: str, tuplst: tuplst_type) -> Any:
     # Try all functions, and test result against predicates
     for func, pred in tuplst:
         try:
@@ -81,12 +104,12 @@ def _(value: str, tuplst):
 # Helper functions that aid with single value no corrections encoding/decoding.
 # Various filters are item dependent.
 #
-def dh_noConv(value, pattern, limit):
+def dh_noConv(value: int, pattern: str, limit: Callable[[int], bool]) -> str:
     """Helper for decoding an integer value, no conversion, no rounding."""
     return dh(value, pattern, lambda i: i, decSinglVal, limit)
 
 
-def dh_dayOfMnth(value, pattern):
+def dh_dayOfMnth(value: int, pattern: str) -> str:
     """
     Helper for decoding a single integer value.
 
@@ -97,7 +120,7 @@ def dh_dayOfMnth(value, pattern):
     return dh_noConv(value, pattern, formatLimits[dayMnthFmts[0]][0])
 
 
-def dh_mnthOfYear(value, pattern):
+def dh_mnthOfYear(value: int, pattern: str) -> str:
     """
     Helper for decoding a single integer value.
 
@@ -107,7 +130,7 @@ def dh_mnthOfYear(value, pattern):
     return dh_noConv(value, pattern, _formatLimit_MonthOfYear[0])
 
 
-def dh_decAD(value, pattern):
+def dh_decAD(value: int, pattern: str) -> str:
     """
     Helper for decoding a single integer value.
 
@@ -117,7 +140,7 @@ def dh_decAD(value, pattern):
               formatLimits['DecadeAD'][0])
 
 
-def dh_decBC(value, pattern):
+def dh_decBC(value: int, pattern: str) -> str:
     """
     Helper for decoding a single integer value.
 
@@ -127,7 +150,7 @@ def dh_decBC(value, pattern):
               formatLimits['DecadeBC'][0])
 
 
-def dh_yearBC(value, pattern):
+def dh_yearBC(value: int, pattern: str) -> str:
     """Helper for decoding a year value.
 
     The value should have no conversion, no rounding, limits to 3000.
@@ -135,7 +158,7 @@ def dh_yearBC(value, pattern):
     return dh_noConv(value, pattern, formatLimits['YearBC'][0])
 
 
-def dh_yearAD(value, pattern):
+def dh_yearAD(value: int, pattern: str) -> str:
     """Helper for decoding a year value.
 
     The value should have no conversion, no rounding, limits to 3000.
@@ -143,7 +166,7 @@ def dh_yearAD(value, pattern):
     return dh_noConv(value, pattern, formatLimits['YearAD'][0])
 
 
-def dh_simpleYearAD(value):
+def dh_simpleYearAD(value: int) -> str:
     """Helper for decoding a single integer value.
 
     This value should be representing a year with no extra symbols.
@@ -151,47 +174,48 @@ def dh_simpleYearAD(value):
     return dh_yearAD(value, '%d')
 
 
-def dh_number(value, pattern):
+def dh_number(value: int, pattern: str) -> str:
     """Helper for decoding a number."""
     return dh_noConv(value, pattern, formatLimits['Number'][0])
 
 
-def dh_centuryAD(value, pattern):
+def dh_centuryAD(value: int, pattern: str) -> str:
     """Helper for decoding an AD century."""
     return dh_noConv(value, pattern, formatLimits['CenturyAD'][0])
 
 
-def dh_centuryBC(value, pattern):
+def dh_centuryBC(value: int, pattern: str) -> str:
     """Helper for decoding an BC century."""
     return dh_noConv(value, pattern, formatLimits['CenturyBC'][0])
 
 
-def dh_millenniumAD(value, pattern):
+def dh_millenniumAD(value: int, pattern: str) -> str:
     """Helper for decoding an AD millennium."""
     return dh_noConv(value, pattern, formatLimits['MillenniumAD'][0])
 
 
-def dh_millenniumBC(value, pattern):
+def dh_millenniumBC(value: int, pattern: str) -> str:
     """Helper for decoding an BC millennium."""
     return dh_noConv(value, pattern, formatLimits['MillenniumBC'][0])
 
 
-def decSinglVal(v):
+def decSinglVal(v: Sequence[Any]) -> Any:
     """Return first item in list v."""
     return v[0]
 
 
-def encDec0(i):
+def encDec0(i: int) -> int:
     """Round to the nearest decade, decade starts with a '0'-ending year."""
     return (i // 10) * 10
 
 
-def encDec1(i):
+def encDec1(i: int) -> int:
     """Round to the nearest decade, decade starts with a '1'-ending year."""
     return encDec0(i) + 1
 
 
-def slh(value, lst):
+@singledispatch
+def slh(value: int, lst: Sequence[str]) -> str:
     """Helper function for simple list value matching.
 
     !!!!! The index starts at 1, so 1st element has index 1, not 0 !!!!!
@@ -207,29 +231,45 @@ def slh(value, lst):
         formats['MonthName']['en']('anything else') => raise ValueError
 
     """
-    return lst.index(value) + 1 if isinstance(value, str) else lst[value - 1]
+    return lst[value - 1]
 
 
-def dh_singVal(value, match):
+@slh.register(str)
+def _(value: str, lst: Sequence[str]) -> int:
+    return lst.index(value) + 1
+
+
+@singledispatch
+def dh_singVal(value: int, match: str) -> str:
     """Helper function to match a single value to a constant."""
     return dh_constVal(value, 0, match)
 
 
-def dh_constVal(value, ind, match):
+@dh_singVal.register(str)
+def _(value: str, match: str) -> int:
+    return dh_constVal(value, 0, match)  # type: ignore[return-value]
+
+
+@singledispatch
+def dh_constVal(value: int, ind: int, match: str) -> str:
     """Helper function to match a single value to a constant.
 
     formats['CurrEvents']['en'](ind) => 'Current Events'
     formats['CurrEvents']['en']('Current Events') => ind
     """
-    if isinstance(value, str):
-        if value == match:
-            return ind
-    elif value == ind:
+    if value == ind:
         return match
     raise ValueError('unknown value {}'.format(value))
 
 
-def alwaysTrue(x):
+@dh_constVal.register(str)
+def _(value: str, ind: int, match: str) -> int:
+    if value == match:
+        return ind
+    raise ValueError('unknown value {}'.format(value))
+
+
+def alwaysTrue(x: Any) -> bool:
     """
     Return True, always.
 
@@ -237,12 +277,11 @@ def alwaysTrue(x):
 
     :param x: not used
     :return: True
-    :rtype: bool
     """
     return True
 
 
-def monthName(lang, ind):
+def monthName(lang: str, ind: int) -> str:
     """Return the month name for a language."""
     return formats['MonthName'][lang](ind)
 
@@ -273,12 +312,14 @@ _guDigitsToLocal = {ord(str(i)): _guDigits[i] for i in range(10)}
 _guLocalToDigits = {ord(_guDigits[i]): str(i) for i in range(10)}
 
 
-def intToLocalDigitsStr(value, digitsToLocalDict):
+def intToLocalDigitsStr(value: int, digitsToLocalDict: Mapping[int, str]
+                        ) -> str:
     """Encode an integer value into a textual form."""
     return str(value).translate(digitsToLocalDict)
 
 
-def localDigitsStrToInt(value, digitsToLocalDict, localToDigitsDict):
+def localDigitsStrToInt(value: str, digitsToLocalDict: Mapping[int, str],
+                        localToDigitsDict: Mapping[int, str]) -> int:
     """Convert digits to integer."""
     # First make sure there are no real digits in the string
     tmp = value.translate(digitsToLocalDict)         # Test
@@ -294,14 +335,14 @@ _romanNumbers = ['-', 'I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII', 'IX',
                  'XXVII', 'XXVIII', 'XXIX', 'XXX']
 
 
-def intToRomanNum(i):
+def intToRomanNum(i: int) -> str:
     """Convert integer to roman numeral."""
     if i >= len(_romanNumbers):
         raise IndexError('Roman value {} is not defined'.format(i))
     return _romanNumbers[i]
 
 
-def romanNumToInt(v):
+def romanNumToInt(v: str) -> int:
     """Convert roman numeral to integer."""
     return _romanNumbers.index(v)
 
@@ -339,7 +380,7 @@ _digitDecoders = {
     # %T is a year in TH: -- all years are shifted: 2005 => 'พ.ศ. 2548'
     'T': (_decimalDigits, lambda v: str(v + 543),
           lambda v: int(v) - 543),
-}
+}  # type: Dict[str, decoder_type]
 
 # Allows to search for '(%%)|(%d)|(%R)|...", and allows one digit 1-9 to set
 # the size of zero-padding for numbers
@@ -350,7 +391,9 @@ _reParameters = re.compile('|'.join('(%[1-9]?{})'.format(s)
 _escPtrnCache2 = {}
 
 
-def escapePattern2(pattern):
+def escapePattern2(pattern: str
+                   ) -> Tuple[Pattern[str], str,
+                              List[Union[decoder_type, decoder_type]]]:
     """
     Convert a string pattern into a regex expression and cache.
 
@@ -358,12 +401,12 @@ def escapePattern2(pattern):
     Returns a compiled regex object and a list of digit decoders.
     """
     @singledispatch
-    def decode(dec: tuple, subpattern: str, newpattern: str,
+    def decode(dec: decoder_type, subpattern: str, newpattern: str,
                strpattern: str) -> Tuple[str, str]:
 
         if len(subpattern) == 3:
             # enforce mandatory field size
-            newpattern += '([%s]{%s})' % (dec[0], subpattern[1])
+            newpattern += '([{}]{{{}}})'.format(dec[0], subpattern[1])
             # add the number of required digits as the last (4th)
             # part of the tuple
             decoders.append(dec + (int(s[1]),))
@@ -390,7 +433,7 @@ def escapePattern2(pattern):
     if pattern not in _escPtrnCache2:
         newPattern = ''  # match starts at the beginning of the string
         strPattern = ''
-        decoders = []
+        decoders = []  # type: List[decoder_type]
         for s in _reParameters.split(pattern):
             if s is None:
                 continue
@@ -412,7 +455,8 @@ def escapePattern2(pattern):
 
 
 @singledispatch
-def dh(value: int, pattern, encf, decf, filter=None):
+def dh(value: int, pattern: str, encf: encf_type, decf: decf_type,
+       filter: Optional[Callable[[int], bool]] = None) -> str:
     """Function to help with year parsing.
 
     Usually it will be used as a lambda call in a map::
@@ -438,7 +482,7 @@ def dh(value: int, pattern, encf, decf, filter=None):
         numeral number.
 
     """
-    compPattern, strPattern, decoders = escapePattern2(pattern)
+    _compPattern, strPattern, decoders = escapePattern2(pattern)
     # Encode an integer value into a textual form.
     # This will be called from outside as well as recursivelly to verify
     # parsed value
@@ -452,19 +496,21 @@ def dh(value: int, pattern, encf, decf, filter=None):
             'parameter count ({}) does not match decoder count ({})'
             .format(len(params), len(decoders)))
         # convert integer parameters into their textual representation
-        params = tuple(_make_parameter(decoders[i], param)
-                       for i, param in enumerate(params))
-        return strPattern % params
+        str_params = tuple(_make_parameter(decoders[i], param)
+                           for i, param in enumerate(params))
+        return strPattern % str_params
     assert len(decoders) == 1, (
         'A single parameter does not match {} decoders.'
         .format(len(decoders)))
     # convert integer parameter into its textual representation
+    assert isinstance(params, int)
     return strPattern % _make_parameter(decoders[0], params)
 
 
 @dh.register(str)
-def _(value: str, pattern, encf, decf, filter=None):
-    compPattern, strPattern, decoders = escapePattern2(pattern)
+def _(value: str, pattern: str, encf: encf_type, decf: decf_type,
+      filter: Optional[Callable[[int], bool]] = None) -> int:
+    compPattern, _strPattern, decoders = escapePattern2(pattern)
     m = compPattern.match(value)
     if m:
         # decode each found value using provided decoder
@@ -483,13 +529,14 @@ def _(value: str, pattern, encf, decf, filter=None):
     raise ValueError("reverse encoding didn't match")
 
 
-def _make_parameter(decoder, param):
+def _make_parameter(decoder: decoder_type, param: int) -> str:
     newValue = decoder[1](param)
-    if len(decoder) == 4 and len(newValue) < decoder[3]:
+    required_digits = decoder[3] if len(decoder) == 4 else None
+    if required_digits is not None and len(newValue) < required_digits:
         # force parameter length by taking the first digit in the list and
         # repeating it required number of times
         # This converts "205" into "0205" for "%4d"
-        newValue = decoder[0][0] * (decoder[3] - len(newValue)) + newValue
+        newValue = decoder[0][0] * (required_digits - len(newValue)) + newValue
     return newValue
 
 
@@ -505,7 +552,7 @@ def _make_parameter(decoder, param):
 # This is useful when trying to decide if a certain article is a localized date
 # or not, or generating dates.
 # See dh() for additional information.
-class MonthNames(Mapping):
+class MonthNames(abc.Mapping):
 
     """A Mapping which reads month names from MediaWiki messages."""
 
@@ -521,7 +568,8 @@ class MonthNames(Mapping):
         'zh': lambda v: slh(v, makeMonthList('%d月')),
     }
 
-    def __getitem__(self, lang):
+    def __getitem__(self, lang: str
+                    ) -> Callable[[int], str]:
         if lang not in self.months:
             site = Site()
             # may_long differs
@@ -533,21 +581,23 @@ class MonthNames(Mapping):
 
         return self.months[lang]
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[str]:
         return iter(self.months)
 
-    def __len__(self):
+    def __len__(self) -> int:
         """Length of preloaded languages with all month names."""
         return len(self.months)
 
 
-class MonthFormat(MutableMapping):
+class MonthFormat(abc.MutableMapping):  # type: ignore[type-arg]
 
     """A Mapping which creates months formats."""
 
     day_formats = {
         'af': ('%d {}', True),
         'ang': ('%d {}', True),
+        'ar': ('%d {}', True),
+        'arz': ('%d {}', True),
         'bg': ('%d {}', False),
         'bn': ('{} %%B', None),
         'ceb': ('{} %d', True),
@@ -596,7 +646,6 @@ class MonthFormat(MutableMapping):
     }
 
     year_formats = {
-        'ar': ('{} %d', None),
         'cs': ('{} %d', None),
         'eo': ('{} de %d', None),
         'es': ('{} de %d', True),
@@ -607,19 +656,17 @@ class MonthFormat(MutableMapping):
         'tl': ('{} %d', None),
     }
 
-    def __init__(self, index, format_key):
+    def __init__(self, index: int, format_key: str) -> None:
         """Initializer of MonthFormat mapping.
 
         :param index: month number
-        :type index: int
         :param format_key: formats key like Day_January or Year_December
-        :type format_key: str
         """
         self.index = index
         self.variant, _, self.month = format_key.partition('_')
-        self.data = {}
+        self.data = {}  # type: Dict[str, Callable[[int], str]]
 
-    def __getitem__(self, key):
+    def __getitem__(self, key: str) -> Callable[[int], str]:
         if key not in self.data:
             if self.variant == 'Day':
                 pattern, ucase = self.day_formats.get(key, ('%d. {}', False))
@@ -642,23 +689,24 @@ class MonthFormat(MutableMapping):
             self.data[key] = eval(expression)
         return self.data[key]
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: str, value: Callable[[int], str]) -> None:
         self.data[key] = value
 
-    def __delitem__(self, key):
+    def __delitem__(self, key: str) -> None:
         raise NotImplementedError("Deleting of key '{}' is not implemented"
                                   .format(key))
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[str]:
         return iter(self.data)
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self.data)
 
 
 formats = {
     'Number': {
         'ar': lambda v: dh_number(v, '%d (عدد)'),
+        'ary': lambda v: dh_number(v, '%d'),
         'be': lambda v: dh_number(v, '%d (лік)'),
         'bg': lambda v: dh_number(v, '%d (число)'),
         'bs': lambda v: dh_number(v, '%d (broj)'),
@@ -717,6 +765,7 @@ formats = {
 
     'YearBC': {
         'af': lambda v: dh_yearBC(v, '%d v.C.'),
+        'ar': lambda v: dh_yearBC(v, '%d ق م'),
         'ast': lambda v: dh_yearBC(v, '%d edC'),
         'be': lambda v: dh_yearBC(v, '%d да н.э.'),
         'bg': lambda v: dh_yearBC(v, '%d г. пр.н.е.'),
@@ -926,6 +975,7 @@ formats = {
     },
 
     'DecadeBC': {
+        'ar': lambda v: dh_decBC(v, 'عقد %d ق م'),
         'de': lambda v: dh_decBC(v, '%der v. Chr.'),
         'da': lambda v: dh_decBC(v, "%d'erne f.Kr."),
         'en': lambda v: dh_decBC(v, '%ds BC'),
@@ -990,7 +1040,7 @@ formats = {
              lambda p: p in (1, 8) or (p >= 20)),
             (lambda v: dh_centuryAD(v, '%dde eeu'), alwaysTrue)]),
         'ang': lambda v: dh_centuryAD(v, '%de gēarhundred'),
-        'ar': lambda v: dh_centuryAD(v, 'قرن %d'),
+        'ar': lambda v: dh_centuryAD(v, 'القرن %d'),
         'ast': lambda v: dh_centuryAD(v, 'Sieglu %R'),
         'be': lambda v: dh_centuryAD(v, '%d стагодзьдзе'),
         'bg': lambda v: dh_centuryAD(v, '%d век'),
@@ -1363,6 +1413,7 @@ formats = {
     },
 
     'Cat_Year_MusicAlbums': {
+        'ar': lambda v: dh_yearAD(v, 'ألبومات %d'),
         'cs': lambda v: dh_yearAD(v, 'Alba roku %d'),
         'en': lambda v: dh_yearAD(v, '%d albums'),
         'fa': lambda v: dh_yearAD(v, 'آلبوم‌های %d (میلادی)'),
@@ -1378,6 +1429,7 @@ formats = {
     'Cat_BirthsAD': {
         'an': lambda v: dh_yearAD(v, '%d (naixencias)'),
         'ar': lambda v: dh_yearAD(v, 'مواليد %d'),
+        'ary': lambda v: dh_yearAD(v, 'زيادة %d'),
         'arz': lambda v: dh_yearAD(v, 'مواليد %d'),
         'bar': lambda v: dh_yearAD(v, 'Geboren %d'),
         'be': lambda v: dh_yearAD(v, 'Нарадзіліся ў %d годзе'),
@@ -1549,10 +1601,14 @@ formats = {
     },
 
     'Cat_BirthsBC': {
+        'ar': lambda v: dh_yearBC(v, 'مواليد %d ق م'),
+        'arz': lambda v: dh_yearBC(v, 'مواليد %d ق م'),
         'en': lambda v: dh_yearBC(v, '%d BC births'),
         'nb': lambda v: dh_yearBC(v, 'Fødsler i %d f.Kr.'),
     },
     'Cat_DeathsBC': {
+        'ar': lambda v: dh_yearBC(v, 'وفيات %d ق م'),
+        'arz': lambda v: dh_yearBC(v, 'وفيات %d ق م'),
         'en': lambda v: dh_yearBC(v, '%d BC deaths'),
         'fr': lambda v: dh_yearBC(v, 'Décès en -%d'),
         'nb': lambda v: dh_yearBC(v, 'Dødsfall i %d f.Kr.'),
@@ -1562,6 +1618,7 @@ formats = {
         'an': lambda v: dh_singVal(v, 'Autualidá'),
         'ang': lambda v: dh_singVal(v, 'Efenealde belimpas'),
         'ar': lambda v: dh_singVal(v, 'أحداث جارية'),
+        'ary': lambda v: dh_singVal(v, 'آخر الأحداث'),
         'arz': lambda v: dh_singVal(v, 'احداث دلوقتى'),
         'be': lambda v: dh_singVal(v, 'Бягучыя падзеі'),
         'bg': lambda v: dh_singVal(v, 'Текущи събития'),
@@ -1617,19 +1674,22 @@ formats = {
         'yo': lambda v: dh_singVal(v, 'Current events'),
         'zh': lambda v: dh_singVal(v, '新闻动态'),
     },
-}
+}  # type: Dict[Union[str, int], Mapping[str, Callable[[int], str]]]
 
 formats['MonthName'] = MonthNames()
 #
 # Add auto-generated empty dictionaries for DayOfMonth and MonthOfYear articles
 #
 for index, day_of_month in enumerate(dayMnthFmts, 1):
-    formats[day_of_month] = MonthFormat(index, day_of_month)
+    val = MonthFormat(index, day_of_month)
+    formats[day_of_month] = val
 for index, month_of_year in enumerate(yrMnthFmts, 1):
-    formats[month_of_year] = MonthFormat(index, month_of_year)
+    val = MonthFormat(index, month_of_year)
+    formats[month_of_year] = val
 
 
-def addFmt1(lang: str, isMnthOfYear, patterns):
+def addFmt1(lang: str, isMnthOfYear: bool,
+            patterns: Sequence[Optional[str]]) -> None:
     """Add 12 month formats for a specific type ('January', 'Feb.').
 
     The function must accept one parameter for the ->int or ->string
@@ -1650,12 +1710,13 @@ def addFmt1(lang: str, isMnthOfYear, patterns):
                     'lambda v: dh_dayOfMnth(v, "{}")'.format(patterns[i]))
 
 
-def makeMonthList(pattern):
+def makeMonthList(pattern: str) -> List[str]:
     """Return a list of 12 elements based on the number of the month."""
     return [pattern % m for m in range(1, 13)]
 
 
-def makeMonthNamedList(lang, pattern, makeUpperCase=None):
+def makeMonthNamedList(lang: str, pattern: str,
+                       makeUpperCase: Optional[bool] = None) -> List[str]:
     """Create a list of 12 elements based on the name of the month.
 
     The language-dependent month name is used as a formatting argument to the
@@ -1786,30 +1847,30 @@ addFmt1('vi', False, makeMonthList('%%d tháng %d'))
 addFmt1('zh', False, makeMonthList('%d月%%d日'))
 
 # Walloon names depend on the day number, thus we must generate various
-# different patterns
+# different patterns:
 
 # For month names beginning with a consonant...
-for i in (0, 1, 2, 4, 5, 6, 8, 10, 11):
-    formats[dayMnthFmts[i]]['wa'] = eval(
-        'lambda m: multi(m, ['
-        '(lambda v: dh_dayOfMnth(v, "%dî d\' {mname}"), lambda p: p == 1), '
-        '(lambda v: dh_dayOfMnth(v, "%d d\' {mname}"), '
-        'lambda p: p in [2,3,20,22,23]), '
-        '(lambda v: dh_dayOfMnth(v, "%d di {mname}"), alwaysTrue)])'
-        .format(mname=waMonthNames[i]))
-
+_consonant_pattern = (
+    'lambda m: multi(m, ['
+    '(lambda v: dh_dayOfMnth(v, "%dî d\' {mname}"), lambda p: p == 1), '
+    '(lambda v: dh_dayOfMnth(v, "%d d\' {mname}"), '
+    'lambda p: p in [2,3,20,22,23]), '
+    '(lambda v: dh_dayOfMnth(v, "%d di {mname}"), alwaysTrue)])'
+)
 # For month names beginning with a vowel...
-for i in (3, 7, 9):
-    formats[dayMnthFmts[i]]['wa'] = eval(
-        'lambda m: multi(m, ['
-        '(lambda v: dh_dayOfMnth(v, "%dî d\' {mname}"), lambda p: p == 1), '
-        '(lambda v: dh_dayOfMnth(v, "%d d\' {mname}"), alwaysTrue)])'
-        .format(mname=waMonthNames[i]))
+_vowel_pattern = (
+    'lambda m: multi(m, ['
+    '(lambda v: dh_dayOfMnth(v, "%dî d\' {mname}"), lambda p: p == 1), '
+    '(lambda v: dh_dayOfMnth(v, "%d d\' {mname}"), alwaysTrue)])'
+)
 
 # Brazil uses '1añ' for the 1st of every month, and number without suffix for
 # all other days
 brMonthNames = makeMonthNamedList('br', '%s', True)
-for i in range(0, 12):
+
+for i in range(12):
+    pattern = _vowel_pattern if i in (3, 7, 9) else _consonant_pattern
+    formats[dayMnthFmts[i]]['wa'] = eval(pattern.format(mname=waMonthNames[i]))
     formats[dayMnthFmts[i]]['br'] = eval(
         'lambda m: multi(m, ['
         '(lambda v: dh_dayOfMnth(v, "%dañ {mname}"), lambda p: p == 1), '
@@ -1891,7 +1952,7 @@ for month in yrMnthFmts:
     formatLimits[month] = _formatLimit_MonthOfYear
 
 
-def _format_limit_dom(days):
+def _format_limit_dom(days: int) -> Tuple[Callable[[int], bool], int, int]:
     """Return day of month format limit."""
     assert 29 <= days <= 31
     return lambda v: 1 <= v <= days, 1, days + 1
@@ -1909,14 +1970,14 @@ for monthId in range(12):
         formatLimits[dayMnthFmts[monthId]] = _format_limit_dom(30)
 
 
-def getAutoFormat(lang, title, ignoreFirstLetterCase=True):
+def getAutoFormat(lang: str, title: str, ignoreFirstLetterCase: bool = True
+                  ) -> Tuple[Optional[str], Optional[str]]:
     """
     Return first matching formatted date value.
 
     :param lang: language code
     :param title: value to format
     :return: dictName ('YearBC', 'December', ...) and value (a year, date, ...)
-    :rtype: tuple
     """
     for dict_name, dictionary in formats.items():
         with suppress(Exception):
@@ -1935,56 +1996,42 @@ def getAutoFormat(lang, title, ignoreFirstLetterCase=True):
     return None, None
 
 
-@deprecated('date.format_date', since='20190526')
-class FormatDate:
-
-    """DEPRECATED. Format a date."""
-
-    def __init__(self, site):
-        """Initializer."""
-        self.site = site
-
-    def __call__(self, m, d):
-        """Return a formatted month and day."""
-        return format_date(m, d, self.site)
-
-
-def format_date(month, day, lang=None, year=2000):
+def format_date(month: int, day: int,
+                lang: Union[None, str, 'pywikibot.site.BaseSite'] = None,
+                year: int = 2000) -> str:
     """Format a date localized to given lang.
 
     :param month: month in range of 1..12
     :param day: day of month in range of 1..31
-    :type day: int
     :param lang: a site object or language key. Defaults to current site.
-    :type lang: BaseSite or string
     :param year: year for which the date is to be formatted. always 29 will be
         given For February except the year is given. Default is leap year 2000.
-    :type year: int
     :return: localized date like "January 11"
-    :rtype: str
     :raises ValueError: Wrong day value; must be 1-28/29/30/31
     :raises IllegalMonthError: bad month number; must be 1-12
     """
     if not lang:
         lang = Site().lang
     elif hasattr(lang, 'lang'):
-        lang = lang.lang
+        lang = lang.lang  # type: ignore[union-attr]
     max_day = calendar.monthrange(year, month)[1]
     if not 1 <= day <= max_day:
         raise ValueError(
             'Wrong day value {day}; must be 1-{max_day}'
             .format(day=day, max_day=max_day))
+    assert isinstance(lang, str)
     return formats[dayMnthFmts[month - 1]][lang](day)
 
 
-def formatYear(lang, year):
+def formatYear(lang: str, year: int) -> str:
     """Return year name in a language."""
     if year < 0:
         return formats['YearBC'][lang](-year)
     return formats['YearAD'][lang](year)
 
 
-def apply_month_delta(date, month_delta=1, add_overlap=False):
+def apply_month_delta(date: datetime.date, month_delta: int = 1,
+                      add_overlap: bool = False) -> datetime.date:
     """
     Add or subtract months from the date.
 
@@ -1999,14 +2046,10 @@ def apply_month_delta(date, month_delta=1, add_overlap=False):
     months are numbered consecutively beginning by 1.
 
     :param date: The starting date
-    :type date: date
     :param month_delta: The amount of months added or subtracted.
-    :type month_delta: int
     :param add_overlap: Add any missing days to the date, increasing the month
         once more.
-    :type add_overlap: bool
     :return: The end date
-    :rtype: type of date
     """
     if not isinstance(month_delta, int):
         raise ValueError('Month delta must be an integer')
@@ -2021,9 +2064,9 @@ def apply_month_delta(date, month_delta=1, add_overlap=False):
     return new_date
 
 
-def get_month_delta(date1, date2):
+def get_month_delta(date1: datetime.date, date2: datetime.date) -> int:
     """
-    Return the difference between to dates in months.
+    Return the difference between two dates in months.
 
     It does only work on calendars with 12 months per year, and where the
     months are consecutive and non-negative numbers.
