@@ -1,7 +1,6 @@
-#!/usr/bin/python
 """Text editor class for your favourite editor."""
 #
-# (C) Pywikibot team, 2004-2020
+# (C) Pywikibot team, 2004-2022
 #
 # Distributed under the terms of the MIT license.
 #
@@ -15,19 +14,22 @@ from typing import Optional
 
 import pywikibot
 from pywikibot import config
+from pywikibot.backports import List, Sequence
 
 
 try:
-    from pywikibot.userinterfaces import gui
+    from pywikibot.userinterfaces import gui  # noqa
+    GUI_ERROR = None
 except ImportError as e:
-    gui = e
+    GUI_ERROR = e
 
 
 class TextEditor:
 
     """Text editor."""
 
-    def _command(self, file_name, text, jump_index=None):
+    def _command(self, file_name: str, text: str,
+                 jump_index: Optional[int] = None) -> List[str]:
         """Return editor selected in user-config.py."""
         if jump_index:
             # Some editors make it possible to mark occurrences of substrings,
@@ -40,6 +42,7 @@ class TextEditor:
             line = column = 0
         # Linux editors. We use startswith() because some users might use
         # parameters.
+        assert config.editor is not None
         if config.editor.startswith('kate'):
             command = ['-l', str(line + 1), '-c', str(column + 1)]
         elif config.editor.startswith('gedit'):
@@ -64,8 +67,8 @@ class TextEditor:
         return command
 
     @staticmethod
-    def _concat(command):
-        return ' '.join("'{}'".format(part) if ' ' in part else part
+    def _concat(command: Sequence[str]) -> str:
+        return ' '.join('{!r}'.format(part) if ' ' in part else part
                         for part in command)
 
     def edit(self, text: str, jumpIndex: Optional[int] = None,
@@ -91,7 +94,7 @@ class TextEditor:
                     tempFile.write(text)
                 creationDate = os.stat(tempFilename).st_mtime
                 cmd = self._command(tempFilename, text, jumpIndex)
-                subprocess.run(cmd, shell=platform == 'win32')
+                subprocess.run(cmd, shell=platform == 'win32', check=True)
                 lastChangeDate = os.stat(tempFilename).st_mtime
                 if lastChangeDate == creationDate:
                     # Nothing changed
@@ -105,13 +108,15 @@ class TextEditor:
                 os.close(handle)
                 os.unlink(tempFilename)
 
-        if isinstance(gui, ImportError):
+        if GUI_ERROR:
             raise ImportError(fill(
                 'Could not load GUI modules: {}. No editor available. '
                 'Set your favourite editor in user-config.py "editor", '
                 'or install python packages tkinter and idlelib, which '
                 'are typically part of Python but may be packaged separately '
-                'on your platform.'.format(gui)) + '\n')
+                'on your platform.'.format(GUI_ERROR)) + '\n')
+
+        assert pywikibot.ui is not None
 
         return pywikibot.ui.editText(text, jumpIndex=jumpIndex,
                                      highlight=highlight)
