@@ -153,18 +153,24 @@ UserWarning: warnings targeted at users
   - ArgumentDeprecationWarning: command line argument problems
   - FamilyMaintenanceWarning: missing information in family definition
 
-*Changed in version 6.0:* exceptions were renamed and are ending with "Error".
+.. versionchanged:: 6.0
+   exceptions were renamed and are ending with "Error".
+
+.. versionchanged:: 7.0
+   All Pywikibot Error exceptions must be imported from
+   ``pywikibot.exceptions``. Deprecated exceptions identifiers were
+   removed.
 """
 #
-# (C) Pywikibot team, 2008-2021
+# (C) Pywikibot team, 2008-2022
 #
 # Distributed under the terms of the MIT license.
 #
 import re
-import sys
-from typing import Optional, Union
+from typing import Any, Optional, Union
 
-from pywikibot.tools import ModuleDeprecationWrapper, issue_deprecation_warning
+import pywikibot
+from pywikibot.tools import issue_deprecation_warning
 from pywikibot.tools._deprecate import _NotImplementedWarning
 
 
@@ -187,7 +193,7 @@ class Error(Exception):
 
     """Pywikibot error."""
 
-    def __init__(self, arg: str):
+    def __init__(self, arg: str) -> None:
         """Initializer."""
         self.unicode = arg
 
@@ -200,19 +206,19 @@ class APIError(Error):
 
     """The wiki site returned an error message."""
 
-    def __init__(self, code, info, **kwargs):
+    def __init__(self, code: str, info: str, **kwargs: Any) -> None:
         """Save error dict returned by MW API."""
         self.code = code
         self.info = info
         self.other = kwargs
         self.unicode = self.__str__()
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """Return internal representation."""
         return '{name}("{code}", "{info}", {other})'.format(
             name=self.__class__.__name__, **self.__dict__)
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Return a string representation."""
         if self.other:
             return '{}: {}\n[{}]'.format(
@@ -229,7 +235,8 @@ class APIMWError(APIError):
 
     """The API site returned an error about a MediaWiki internal exception."""
 
-    def __init__(self, mediawiki_exception_class_name, info, **kwargs):
+    def __init__(self, mediawiki_exception_class_name: str, info: str,
+                 **kwargs: Any) -> None:
         """Save error dict returned by MW API."""
         self.mediawiki_exception_class_name = mediawiki_exception_class_name
         code = 'internal_api_error_' + mediawiki_exception_class_name
@@ -240,9 +247,9 @@ class UploadError(APIError):
 
     """Upload failed with a warning message (passed as the argument)."""
 
-    def __init__(self, code, message,
+    def __init__(self, code: str, message: str,
                  file_key: Optional[str] = None,
-                 offset: Union[int, bool] = 0):
+                 offset: Union[int, bool] = 0) -> None:
         """
         Create a new UploadError instance.
 
@@ -256,7 +263,7 @@ class UploadError(APIError):
         self.offset = offset
 
     @property
-    def message(self):
+    def message(self) -> str:
         """Return warning message."""
         return self.info
 
@@ -274,12 +281,12 @@ class PageRelatedError(Error):
     # Override this in subclasses.
     message = ''
 
-    def __init__(self, page, message: Optional[str] = None):
+    def __init__(self, page: 'pywikibot.page.BasePage',
+                 message: Optional[str] = None) -> None:
         """
         Initializer.
 
         :param page: Page that caused the exception
-        :type page: Page object
         """
         if message:
             self.message = message
@@ -296,7 +303,7 @@ class PageRelatedError(Error):
         elif re.search(r'%\(\w+\)s', self.message):
             issue_deprecation_warning("'%' style messages are deprecated, "
                                       'please use str.format() style instead',
-                                      since='20210504')
+                                      since='6.2.0')
             msg = self.message % self.__dict__
         elif '%s' in self.message:
             msg = self.message % page
@@ -319,7 +326,8 @@ class OtherPageSaveError(PageSaveRelatedError):
 
     message = 'Edit to page {title} failed:\n{reason}'
 
-    def __init__(self, page, reason: Union[str, Exception]):
+    def __init__(self, page: 'pywikibot.page.BasePage',
+                 reason: Union[str, Exception]) -> None:
         """Initializer.
 
         :param reason: Details of the problem
@@ -328,7 +336,7 @@ class OtherPageSaveError(PageSaveRelatedError):
         super().__init__(page)
 
     @property
-    def args(self):
+    def args(self) -> str:  # type: ignore[override]
         """Expose args."""
         return str(self.reason)
 
@@ -371,11 +379,10 @@ class InconsistentTitleError(PageLoadRelatedError):
 
     """Page receives a title inconsistent with query."""
 
-    def __init__(self, page, actual: str):
+    def __init__(self, page: 'pywikibot.page.BasePage', actual: str) -> None:
         """Initializer.
 
         :param page: Page that caused the exception
-        :type page: Page object
         :param actual: title obtained by query
 
         """
@@ -448,11 +455,11 @@ class InterwikiRedirectPageError(PageRelatedError):
                'Page: {page}\n'
                'Target page: {target_page} on {target_site}.')
 
-    def __init__(self, page, target_page):
+    def __init__(self, page: 'pywikibot.page.BasePage',
+                 target_page: 'pywikibot.page.BasePage') -> None:
         """Initializer.
 
         :param target_page: Target page of the redirect.
-        :type target_page: Page
         """
         self.target_page = target_page
         self.target_site = target_page.site
@@ -463,10 +470,10 @@ class InvalidPageError(PageLoadRelatedError):
 
     """Missing page history.
 
-    *New in version 6.2.*
+    .. versionadded:: 6.2
     """
 
-    message = 'Page %s is invalid.'
+    message = 'Page {} is invalid.'
 
 
 class InvalidTitleError(Error):
@@ -499,8 +506,6 @@ class SectionError(Error):
 
     """The section specified by # does not exist."""
 
-    pass
-
 
 class NoCreateError(PageSaveRelatedError):
 
@@ -542,13 +547,12 @@ class AbuseFilterDisallowedError(PageSaveRelatedError):
 
     """Page save failed because the AbuseFilter disallowed it."""
 
-    message = ('Edit to page %(title)s disallowed by the AbuseFilter.\n'
-               '%(info)s')
+    message = ('Edit to page {title} disallowed by the AbuseFilter.\n'
+               '{info}')
 
-    def __init__(self, page, info, other):
+    def __init__(self, page: 'pywikibot.page.BasePage', info: str) -> None:
         """Initializer."""
         self.info = info
-        self.other = other
         super().__init__(page)
 
 
@@ -559,7 +563,7 @@ class SpamblacklistError(PageSaveRelatedError):
     message = ('Edit to page {title} rejected by spam filter due to '
                'content:\n{url}')
 
-    def __init__(self, page, url):
+    def __init__(self, page: 'pywikibot.page.BasePage', url: str) -> None:
         """Initializer."""
         self.url = url
         super().__init__(page)
@@ -648,12 +652,11 @@ class NoWikibaseEntityError(WikiBaseError):
 
     """This entity doesn't exist."""
 
-    def __init__(self, entity):
+    def __init__(self, entity: 'pywikibot.page.WikibaseEntity') -> None:
         """
         Initializer.
 
         :param entity: Wikibase entity
-        :type entity: WikibaseEntity
         """
         super().__init__("Entity '{}' doesn't exist on {}"
                          .format(entity.id, entity.repo))
@@ -678,39 +681,3 @@ class TimeoutError(Error):
 class MaxlagTimeoutError(TimeoutError):
 
     """Request failed with a maxlag timeout error."""
-
-
-DEPRECATED_EXCEPTIONS = {
-    'NoUsername': 'NoUsernameError',
-    'NoPage': 'NoPageError',
-    'UnsupportedPage': 'UnsupportedPageError',
-    'NoMoveTarget': 'NoMoveTargetError',
-    'InconsistentTitleReceived': 'InconsistentTitleError',
-    'UnknownSite': 'UnknownSiteError',
-    'UnknownFamily': 'UnknownFamilyError',
-    'UnknownExtension': 'UnknownExtensionError',
-    'IsRedirectPage': 'IsRedirectPageError',
-    'IsNotRedirectPage': 'IsNotRedirectPageError',
-    'CircularRedirect': 'CircularRedirectError',
-    'InterwikiRedirectPage': 'InterwikiRedirectPageError',
-    'InvalidTitle': 'InvalidTitleError',
-    'LockedPage': 'LockedPageError',
-    'LockedNoPage': 'LockedNoPageError',
-    'CascadeLockedPage': 'CascadeLockedPageError',
-    'EditConflict': 'EditConflictError',
-    'PageDeletedConflict': 'PageDeletedConflictError',
-    'PageCreatedConflict': 'PageCreatedConflictError',
-    'ArticleExistsConflict': 'ArticleExistsConflictError',
-    'AutoblockUser': 'AutoblockUserError',
-    'NoWikibaseEntity': 'NoWikibaseEntityError',
-    'CoordinateGlobeUnknownException': 'CoordinateGlobeUnknownError',
-    'EntityTypeUnknownException': 'EntityTypeUnknownError',
-}
-
-wrapper = ModuleDeprecationWrapper(__name__)
-module = sys.modules[__name__]
-
-for old_name, new_name in DEPRECATED_EXCEPTIONS.items():
-    setattr(module, old_name, getattr(module, new_name))
-    wrapper.add_deprecated_attr(old_name, replacement_name=new_name,
-                                since='20210423')
