@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Family module for Wikimedia chapter, thematic organisation and WUG wikis."""
 #
 # (C) Pywikibot team, 2012-2020
@@ -6,7 +5,6 @@
 # Distributed under the terms of the MIT license.
 #
 from pywikibot import family
-from pywikibot.tools import deprecated, classproperty
 
 
 class Family(family.SubdomainFamily, family.WikimediaFamily):
@@ -26,9 +24,3 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
         'nl', 'no', 'nyc', 'pl', 'pt', 'punjabi', 'romd', 'rs', 'ru', 'se',
         'tr', 'ua', 'uk', 've', 'wb',
     ]
-
-    @classproperty
-    @deprecated(since='20150621')
-    def countries(cls):
-        """Deprecated."""
-        return cls.codes
