@@ -1,6 +1,6 @@
 """Family module for Wikipedia."""
 #
-# (C) Pywikibot team, 2004-2021
+# (C) Pywikibot team, 2004-2022
 #
 # Distributed under the terms of the MIT license.
 #
@@ -15,50 +15,48 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
     name = 'wikipedia'
 
     closed_wikis = [
-        # See:
         # https://noc.wikimedia.org/conf/highlight.php?file=dblists/closed.dblist
         'aa', 'cho', 'ho', 'hz', 'ii', 'kj', 'kr', 'lrc', 'mh', 'mus', 'ng',
         'ten',
     ]
 
     removed_wikis = [
-        # See:
         # https://noc.wikimedia.org/conf/highlight.php?file=dblists/deleted.dblist
-        'dk', 'ru-sib', 'tlh', 'tokipona', 'zh_cn', 'zh_tw',
+        'dk', 'mo', 'ru-sib', 'tlh', 'tokipona', 'zh_cn', 'zh_tw',
     ]
 
     languages_by_size = [
-        'en', 'ceb', 'sv', 'de', 'fr', 'nl', 'ru', 'it', 'es', 'pl', 'arz',
-        'ja', 'vi', 'war', 'zh', 'ar', 'uk', 'pt', 'fa', 'ca', 'sr', 'id',
-        'no', 'ko', 'fi', 'hu', 'cs', 'sh', 'zh-min-nan', 'ro', 'tr', 'eu',
-        'ce', 'ms', 'tt', 'eo', 'he', 'hy', 'bg', 'da', 'azb', 'sk', 'kk',
-        'min', 'et', 'hr', 'be', 'lt', 'el', 'simple', 'az', 'gl', 'sl', 'ur',
-        'nn', 'ka', 'hi', 'uz', 'ta', 'th', 'la', 'cy', 'ast', 'vo', 'mk',
-        'zh-yue', 'bn', 'lv', 'tg', 'my', 'af', 'mg', 'oc', 'bs', 'sq', 'nds',
-        'ky', 'mr', 'be-tarask', 'ml', 'new', 'te', 'br', 'vec', 'pms', 'sw',
-        'jv', 'pnb', 'ht', 'su', 'lb', 'ba', 'ga', 'szl', 'is', 'tl', 'lmo',
-        'cv', 'fy', 'wuu', 'sco', 'an', 'diq', 'ku', 'pa', 'yo', 'ckb', 'ne',
-        'bar', 'io', 'gu', 'als', 'kn', 'scn', 'bpy', 'ia', 'qu', 'mn', 'si',
-        'bat-smg', 'nv', 'xmf', 'or', 'cdo', 'ilo', 'gd', 'yi', 'am', 'avk',
-        'nap', 'sd', 'bug', 'hsb', 'mai', 'frr', 'fo', 'os', 'li', 'map-bms',
-        'mzn', 'sah', 'eml', 'ps', 'ace', 'gor', 'sa', 'bcl', 'wa', 'crh',
-        'zh-classical', 'lij', 'mrj', 'mhr', 'ha', 'hif', 'hak', 'roa-tara',
-        'shn', 'hyw', 'pam', 'as', 'zu', 'nso', 'km', 'ban', 'ie', 'rue', 'se',
-        'bh', 'vls', 'nds-nl', 'mi', 'sn', 'sc', 'nah', 'vep', 'gan', 'myv',
-        'glk', 'kab', 'sat', 'so', 'ab', 'bo', 'tk', 'co', 'fiu-vro', 'kv',
-        'csb', 'pcd', 'frp', 'udm', 'ug', 'gv', 'ay', 'zea', 'kw', 'nrm',
-        'lez', 'ary', 'gn', 'lfn', 'mni', 'stq', 'mt', 'mwl', 'lo', 'rm',
-        'skr', 'bjn', 'olo', 'lad', 'gom', 'koi', 'fur', 'ang', 'dsb', 'dty',
-        'ext', 'ln', 'tyv', 'cbk-zam', 'smn', 'dv', 'ksh', 'gag', 'pfl', 'pi',
-        'pag', 'av', 'bxr', 'awa', 'tay', 'haw', 'pap', 'ig', 'rw', 'xal',
-        'krc', 'za', 'szy', 'pdc', 'kaa', 'inh', 'arc', 'to', 'nov', 'kbp',
-        'jam', 'wo', 'tpi', 'na', 'kbd', 'atj', 'ki', 'tet', 'tcy', 'ak',
-        'lld', 'lg', 'jbo', 'bi', 'roa-rup', 'lbe', 'kg', 'ty', 'mdf', 'xh',
-        'fj', 'srn', 'om', 'trv', 'gcr', 'sm', 'ltg', 'chr', 'nia', 'nqo',
-        'mnw', 'pih', 'got', 'st', 'kl', 'mad', 'tw', 'cu', 'ny', 'tn', 'ts',
-        'bm', 'rmy', 've', 'chy', 'rn', 'tum', 'iu', 'ss', 'ks', 'ch', 'alt',
-        'pnt', 'ady', 'ee', 'ff', 'din', 'ik', 'sg', 'dz', 'ti', 'cr', 'dag',
-        'shi',
+        'en', 'ceb', 'de', 'sv', 'fr', 'nl', 'ru', 'es', 'it', 'arz', 'pl',
+        'ja', 'vi', 'zh', 'war', 'ar', 'uk', 'pt', 'fa', 'ca', 'sr', 'id',
+        'ko', 'no', 'fi', 'hu', 'cs', 'tr', 'ce', 'sh', 'zh-min-nan', 'ro',
+        'tt', 'eu', 'ms', 'eo', 'he', 'hy', 'bg', 'da', 'azb', 'sk', 'kk',
+        'et', 'min', 'be', 'hr', 'el', 'simple', 'lt', 'az', 'gl', 'sl', 'ur',
+        'nn', 'ka', 'hi', 'th', 'ta', 'uz', 'la', 'cy', 'ast', 'mk', 'vo',
+        'zh-yue', 'bn', 'lv', 'tg', 'my', 'af', 'mg', 'bs', 'oc', 'sq', 'mr',
+        'nds', 'ky', 'ml', 'be-tarask', 'te', 'new', 'br', 'sw', 'vec', 'jv',
+        'pms', 'pnb', 'ht', 'su', 'lb', 'ba', 'ga', 'szl', 'is', 'ku', 'lmo',
+        'cv', 'fy', 'tl', 'wuu', 'an', 'ckb', 'sco', 'diq', 'pa', 'yo', 'ne',
+        'bar', 'io', 'gu', 'als', 'kn', 'scn', 'bpy', 'ia', 'qu', 'avk', 'mn',
+        'nv', 'xmf', 'crh', 'si', 'lld', 'bat-smg', 'or', 'gd', 'frr', 'os',
+        'cdo', 'yi', 'ilo', 'sd', 'bug', 'ha', 'am', 'nap', 'sah', 'ps', 'hsb',
+        'map-bms', 'fo', 'mai', 'li', 'mzn', 'ban', 'eml', 'gor', 'ace', 'bcl',
+        'sa', 'wa', 'zh-classical', 'lij', 'shn', 'zu', 'mrj', 'mhr', 'as',
+        'hif', 'mni', 'hyw', 'hak', 'roa-tara', 'km', 'pam', 'ie', 'nso',
+        'rue', 'so', 'sn', 'bh', 'se', 'vls', 'nds-nl', 'myv', 'mi', 'nah',
+        'sc', 'sat', 'vep', 'gan', 'kab', 'glk', 'tk', 'fiu-vro', 'co', 'bo',
+        'ab', 'frp', 'kv', 'csb', 'pcd', 'kw', 'gv', 'ug', 'ary', 'udm', 'ay',
+        'skr', 'nrm', 'zea', 'gn', 'mt', 'bjn', 'lez', 'smn', 'lfn', 'stq',
+        'ig', 'lo', 'mwl', 'olo', 'rm', 'fur', 'lad', 'ang', 'gom', 'koi',
+        'ext', 'tyv', 'dsb', 'dty', 'ln', 'cbk-zam', 'rw', 'dv', 'ksh', 'gag',
+        'bxr', 'av', 'pfl', 'pag', 'pi', 'haw', 'awa', 'tay', 'pap', 'krc',
+        'xal', 'szy', 'inh', 'za', 'kaa', 'pdc', 'atj', 'tw', 'to', 'tum',
+        'arc', 'dag', 'tpi', 'kbp', 'jam', 'na', 'mdf', 'wo', 'kbd', 'tcy',
+        'nov', 'nia', 'ki', 'tet', 'lg', 'bi', 'jbo', 'roa-rup', 'fj', 'kg',
+        'lbe', 'xh', 'nqo', 'ty', 'mnw', 'ks', 'cu', 'shi', 'trv', 'srn', 'om',
+        'sm', 'gcr', 'alt', 'chr', 'ltg', 'pih', 'ny', 'got', 'mad', 'st',
+        'ami', 'rmy', 'guw', 'tn', 'bm', 'ts', 'chy', 've', 'rn', 'iu', 'ak',
+        'ss', 'ch', 'pnt', 'ady', 'ik', 'ff', 'ee', 'din', 'sg', 'kl', 'pwn',
+        'ti', 'dz', 'cr',
     ]
 
     # Sites we want to edit but not count as real languages
@@ -198,6 +196,7 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
         'fr': ('En cours',),
         'he': ('בעבודה',),
         'hr': ('Radovi',),
+        'hy': ('Խմբագրում եմ',),
         'sr': ('Радови у току', 'Рут',),
         'test': ('In use',),
         'ur': ('زیر ترمیم',),
