@@ -1,6 +1,6 @@
 """Family module for Wikisource."""
 #
-# (C) Pywikibot team, 2004-2021
+# (C) Pywikibot team, 2004-2022
 #
 # Distributed under the terms of the MIT license.
 #
@@ -16,22 +16,22 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
     name = 'wikisource'
 
     closed_wikis = [
-        # https://noc.wikimedia.org/conf/highlight.php?file=dblists/closed.dblist  # noqa
+        # https://noc.wikimedia.org/conf/highlight.php?file=dblists/closed.dblist
         'ang', 'ht',
     ]
     removed_wikis = [
-        # https://noc.wikimedia.org/conf/highlight.php?file=dblists/deleted.dblist  # noqa
+        # https://noc.wikimedia.org/conf/highlight.php?file=dblists/deleted.dblist
         'tokipona',
     ]
 
     languages_by_size = [
         'pl', 'en', 'ru', 'de', 'fr', 'zh', 'he', 'it', 'es', 'ar', 'uk', 'cs',
-        'gu', 'mul', 'pt', 'sr', 'fa', 'hu', 'sv', 'ko', 'ml', 'sa', 'bn',
-        'te', 'ta', 'sl', 'hy', 'el', 'tr', 'ro', 'fi', 'ja', 'vi', 'nap',
-        'az', 'la', 'ca', 'br', 'nl', 'kn', 'hr', 'th', 'no', 'eo', 'is',
-        'vec', 'pms', 'id', 'hi', 'be', 'et', 'lij', 'da', 'mk', 'yi', 'as',
-        'mr', 'bg', 'li', 'lt', 'wa', 'eu', 'or', 'pa', 'cy', 'gl', 'bs',
-        'sah', 'sk', 'zh-min-nan', 'fo', 'ban',
+        'mul', 'gu', 'sr', 'pt', 'sv', 'fa', 'hu', 'ko', 'ml', 'bn', 'sa',
+        'ta', 'te', 'sl', 'tr', 'hy', 'el', 'la', 'vi', 'ja', 'ro', 'fi',
+        'nap', 'nl', 'az', 'ca', 'br', 'kn', 'hr', 'no', 'id', 'th', 'be',
+        'hi', 'eo', 'is', 'vec', 'pms', 'ban', 'da', 'lij', 'et', 'mk', 'mr',
+        'cy', 'yi', 'as', 'bg', 'li', 'lt', 'jv', 'wa', 'pa', 'eu', 'or', 'gl',
+        'bs', 'sah', 'sk', 'zh-min-nan', 'fo',
     ]
 
     # Sites we want to edit but not count as real languages
@@ -51,7 +51,7 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
     # the main page, so using 'wikisource.org'
     @classproperty
     def langs(cls):
-        cls.langs = super(Family, cls).langs
+        cls.langs = super().langs
         cls.langs['mul'] = cls.domain
         cls.langs['beta'] = 'en.wikisource.beta.wmflabs.org'
         return cls.langs
@@ -59,7 +59,7 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
     # Need to explicitly inject the beta domain
     @classproperty
     def domains(cls):
-        cls.domains = super(Family, cls).domains
+        cls.domains = super().domains
         cls.domains.append(cls.langs['beta'])
         return cls.domains
 
@@ -67,7 +67,7 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
     # the main page, so using mul alias, see T114574 and T241413
     @classproperty
     def code_aliases(cls):
-        cls.code_aliases = super(Family, cls).code_aliases.copy()
+        cls.code_aliases = super().code_aliases.copy()
         aliases = cls.alphabetic + ['-', 'www']
         for code in aliases:
             if (code not in cls.languages_by_size
