@@ -1,7 +1,6 @@
-#!/usr/bin/python
 """Library to log the bot in to a wiki account."""
 #
-# (C) Pywikibot team, 2003-2021
+# (C) Pywikibot team, 2003-2022
 #
 # Distributed under the terms of the MIT license.
 #
@@ -9,21 +8,15 @@ import codecs
 import os
 import webbrowser
 from enum import IntEnum
-from typing import Optional
+from typing import Any, Optional
 from warnings import warn
 
 import pywikibot
-import pywikibot.data.api
 from pywikibot import __url__, config
+from pywikibot.backports import Dict, Tuple
 from pywikibot.comms import http
 from pywikibot.exceptions import APIError, NoUsernameError
-from pywikibot.tools import (
-    ModuleDeprecationWrapper,
-    deprecated_args,
-    file_mode_checker,
-    normalize_username,
-    remove_last_args,
-)
+from pywikibot.tools import file_mode_checker, normalize_username
 
 
 try:
@@ -37,9 +30,6 @@ class _PasswordFileWarning(UserWarning):
     """The format of password file is incorrect."""
 
 
-_logger = 'wiki.login'
-
-
 # On some wikis you are only allowed to run a bot if there is a link to
 # the bot's user page in a specific list.
 # If bots are listed in a template, the templates name must be given as
@@ -75,7 +65,7 @@ class LoginStatus(IntEnum):
     NOT_LOGGED_IN = -1
     AS_USER = 0
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """Return internal representation."""
         return 'LoginStatus({})'.format(self)
 
@@ -84,16 +74,15 @@ class LoginManager:
 
     """Site login manager."""
 
-    @deprecated_args(username='user', verbose=True, sysop=True)
     def __init__(self, password: Optional[str] = None,
-                 site=None, user: Optional[str] = None):
+                 site: Optional['pywikibot.site.BaseSite'] = None,
+                 user: Optional[str] = None) -> None:
         """
         Initializer.
 
         All parameters default to defaults in user-config.
 
         :param site: Site object to log into
-        :type site: BaseSite
         :param user: username to use.
             If user is None, the username is loaded from config.usernames.
         :param password: password to use
@@ -122,11 +111,11 @@ class LoginManager:
         if getattr(config, 'password_file', ''):
             self.readPassword()
 
-    def check_user_exists(self):
+    def check_user_exists(self) -> None:
         """
         Check that the username exists on the site.
 
-        :see: https://www.mediawiki.org/wiki/API:Users
+        .. seealso:: :api:`Users`
 
         :raises pywikibot.exceptions.NoUsernameError: Username doesn't exist in
             user list.
@@ -156,7 +145,7 @@ class LoginManager:
             raise NoUsernameError("Username '{}' does not exist on {}"
                                   .format(main_username, self.site))
 
-    def botAllowed(self):
+    def botAllowed(self) -> bool:
         """
         Check whether the bot is listed on a specific page.
 
@@ -180,17 +169,16 @@ class LoginManager:
         # No bot policies on other sites
         return True
 
-    def login_to_site(self):
+    def login_to_site(self) -> None:
         """Login to the site."""
         # THIS IS OVERRIDDEN IN data/api.py
         raise NotImplementedError
 
-    @remove_last_args(['data'])
     def storecookiedata(self) -> None:
         """Store cookie data."""
         http.cookie_jar.save(ignore_discard=True)
 
-    def readPassword(self):
+    def readPassword(self) -> None:
         """
         Read passwords from a file.
 
@@ -218,6 +206,7 @@ class LoginManager:
         """
         # Set path to password file relative to the user_config
         # but fall back on absolute path for backwards compatibility
+        assert config.base_dir is not None and config.password_file is not None
         password_file = os.path.join(config.base_dir, config.password_file)
         if not os.path.isfile(password_file):
             password_file = config.password_file
@@ -272,18 +261,15 @@ class LoginManager:
         'FAIL': 'does not have read permissions',
     }
 
-    def login(self, retry=False, autocreate=False) -> bool:
+    def login(self, retry: bool = False, autocreate: bool = False) -> bool:
         """
         Attempt to log into the server.
 
-        :see: https://www.mediawiki.org/wiki/API:Login
+        .. seealso:: :api:`Login`
 
         :param retry: infinitely retry if the API returns an unknown error
-        :type retry: bool
-
         :param autocreate: if true, allow auto-creation of the account
                            using unified login
-        :type autocreate: bool
 
         :raises pywikibot.exceptions.NoUsernameError: Username is not
             recognised by the site.
@@ -332,7 +318,7 @@ class BotPassword:
 
     """BotPassword object for storage in password file."""
 
-    def __init__(self, suffix: str, password: str):
+    def __init__(self, suffix: str, password: str) -> None:
         """
         Initializer.
 
@@ -366,16 +352,15 @@ class OauthLoginManager(LoginManager):
     # NOTE: Currently OauthLoginManager use mwoauth directly to complete OAuth
     # authentication process
 
-    @deprecated_args(sysop=True)
-    def __init__(self, password: Optional[str] = None, site=None,
-                 user: Optional[str] = None):
+    def __init__(self, password: Optional[str] = None,
+                 site: Optional['pywikibot.site.BaseSite'] = None,
+                 user: Optional[str] = None) -> None:
         """
         Initializer.
 
         All parameters default to defaults in user-config.
 
         :param site: Site object to log into
-        :type site: BaseSite
         :param user: consumer key
         :param password: consumer secret
 
@@ -393,19 +378,17 @@ class OauthLoginManager(LoginManager):
                            'should be removed if OAuth enabled.'
                            .format(login=self))
         self._consumer_token = (user, password)
-        self._access_token = None
+        self._access_token = None  # type: Optional[Tuple[str, str]]
 
-    def login(self, retry=False, force=False):
+    def login(self, retry: bool = False, force: bool = False) -> bool:
         """
         Attempt to log into the server.
 
-        :see: https://www.mediawiki.org/wiki/API:Login
+        .. seealso:: :api:`Login`
 
         :param retry: infinitely retry if exception occurs during
             authentication.
-        :type retry: bool
         :param force: force to re-authenticate
-        :type force: bool
         """
         if self.access_token is None or force:
             pywikibot.output(
@@ -424,39 +407,38 @@ class OauthLoginManager(LoginManager):
                 request_qs = pywikibot.input('Response query string: ')
                 access_token = handshaker.complete(request_token, request_qs)
                 self._access_token = (access_token.key, access_token.secret)
+                return True
             except Exception as e:
                 pywikibot.error(e)
                 if retry:
-                    self.login(retry=True, force=force)
+                    return self.login(retry=True, force=force)
+                return False
         else:
             pywikibot.output('Logged in to {site} via consumer {key}'
                              .format(key=self.consumer_token[0],
                                      site=self.site))
+            return True
 
     @property
-    def consumer_token(self):
+    def consumer_token(self) -> Tuple[str, str]:
         """
         Return OAuth consumer key token and secret token.
 
-        :see: https://www.mediawiki.org/wiki/API:Tokens
-
-        :rtype: tuple of two str
+        .. seealso:: :api:`Tokens`
         """
         return self._consumer_token
 
     @property
-    def access_token(self):
+    def access_token(self) -> Optional[Tuple[str, str]]:
         """
         Return OAuth access key token and secret token.
 
-        :see: https://www.mediawiki.org/wiki/API:Tokens
-
-        :rtype: tuple of two str
+        .. seealso:: :api:`Tokens`
         """
         return self._access_token
 
     @property
-    def identity(self) -> Optional[dict]:
+    def identity(self) -> Optional[Dict[str, Any]]:
         """Get identifying information about a user via an authorized token."""
         if self.access_token is None:
             pywikibot.error('Access token not set')
@@ -471,12 +453,3 @@ class OauthLoginManager(LoginManager):
         except Exception as e:
             pywikibot.error(e)
             return None
-
-
-OAuthImpossible = ImportError
-
-wrapper = ModuleDeprecationWrapper(__name__)
-wrapper.add_deprecated_attr(
-    'OAuthImpossible',
-    replacement_name='ImportError',
-    since='20210423')
