@@ -1,10 +1,9 @@
-#!/usr/bin/python
 """Special bot library containing UploadRobot.
 
 Do not import classes directly from here but from specialbots.
 """
 #
-# (C) Pywikibot team, 2003-2021
+# (C) Pywikibot team, 2003-2022
 #
 # Distributed under the terms of the MIT license.
 #
@@ -20,23 +19,16 @@ import requests
 
 import pywikibot
 import pywikibot.comms.http as http
-import pywikibot.data.api
 from pywikibot import config
 from pywikibot.backports import List
 from pywikibot.bot import BaseBot, QuitKeyboardInterrupt
 from pywikibot.exceptions import APIError, FatalServerError, NoPageError
-from pywikibot.tools import deprecated_args
-from pywikibot.tools.formatter import color_format
 
 
 class UploadRobot(BaseBot):
 
     """Upload bot."""
 
-    @deprecated_args(urlEncoding='url_encoding', useFilename='use_filename',
-                     keepFilename='keep_filename',
-                     verifyDescription='verify_description',
-                     ignoreWarning='ignore_warning', targetSite='target_site')
     def __init__(self, url: Union[List[str], str], *,
                  url_encoding=None,
                  description: str = '',
@@ -51,13 +43,14 @@ class UploadRobot(BaseBot):
                  summary: Optional[str] = None,
                  filename_prefix: Optional[str] = None,
                  force_if_shared: bool = False,
-                 **kwargs):
+                 **kwargs) -> None:
         """Initializer.
 
-        *Changed in version 6.2:* asynchronous upload is used if
-        *asynchronous* parameter is set.
+        .. versionchanged:: 6.2
+           asynchronous upload is used if *asynchronous* parameter is set
 
-        *New in version 6.4:* force_if_shared parameter.
+        .. versionchanged:: 6.4
+           *force_if_shared* parameter was added
 
         :param url: path to url or local file, or list of urls or paths
             to local files.
@@ -196,9 +189,8 @@ class UploadRobot(BaseBot):
         :return: False if this warning should cause an abort, True if it should
             be ignored or None if this warning has no default handler.
         """
-        if self.aborts is not True:
-            if warning in self.aborts:
-                return False
+        if self.aborts is not True and warning in self.aborts:
+            return False
         if self.ignore_warning is True or (self.ignore_warning is not False
                                            and warning in self.ignore_warning):
             return True
@@ -248,7 +240,7 @@ class UploadRobot(BaseBot):
                 filename = newfn
         # FIXME: these 2 belong somewhere else, presumably in family
         # forbidden characters are handled by pywikibot/page.py
-        forbidden = ':*?/\\'  # to be extended
+        forbidden = ':*/\\'  # to be extended
         try:
             allowed_formats = self.target_site.siteinfo.get(
                 'fileextensions', get_default=False)
@@ -334,9 +326,8 @@ class UploadRobot(BaseBot):
 
         while not self.description or self.verify_description:
             if not self.description:
-                pywikibot.output(color_format(
-                    '{lightred}It is not possible to upload a file '
-                    'without a description.{default}'))
+                pywikibot.output('<<lightred>>It is not possible to upload a '
+                                 'file without a description.<<default>>')
             assert not self.opt.always
             # if no description, ask if user want to add one or quit,
             # and loop until one is filled.
@@ -378,15 +369,19 @@ class UploadRobot(BaseBot):
         """
         return self.ignore_warning is True or warn_code in self.ignore_warning
 
-    def upload_file(self, file_url, _file_key=None, _offset=0):
+    def upload_file(self, file_url):
         """
         Upload the image at file_url to the target wiki.
 
-        :see: https://www.mediawiki.org/wiki/API:Upload
+        .. seealso:: :api:`Upload`
 
         Return the filename that was used to upload the image.
         If the upload fails, ask the user whether to try again or not.
         If the user chooses not to retry, return None.
+
+        .. versionchanged:: 7.0
+           If 'copyuploadbaddomain' API error occurred in first step,
+           download the file and upload it afterwards
         """
         filename = self.process_filename(file_url)
         if not filename:
@@ -399,38 +394,51 @@ class UploadRobot(BaseBot):
         pywikibot.output('Uploading file to {}...'.format(site))
 
         ignore_warnings = self.ignore_warning is True or self._handle_warnings
-        if '://' in file_url and not site.has_right('upload_by_url'):
-            try:
-                file_url = self.read_file_content(file_url)
-            except FatalServerError:
-                return None
 
-        try:
-            success = imagepage.upload(file_url,
-                                       ignore_warnings=ignore_warnings,
-                                       chunk_size=self.chunk_size,
-                                       _file_key=_file_key, _offset=_offset,
-                                       asynchronous=self.asynchronous,
-                                       comment=self.summary)
-        except APIError as error:
-            if error.code == 'uploaddisabled':
-                pywikibot.error(
-                    'Upload error: Local file uploads are disabled on {}.'
-                    .format(site))
-            else:
+        download = False
+        while True:
+            if '://' in file_url \
+               and (not site.has_right('upload_by_url') or download):
+                try:
+                    file_url = self.read_file_content(file_url)
+                except FatalServerError:
+                    pywikibot.exception()
+                    return None
+
+            try:
+                success = imagepage.upload(file_url,
+                                           ignore_warnings=ignore_warnings,
+                                           chunk_size=self.chunk_size,
+                                           asynchronous=self.asynchronous,
+                                           comment=self.summary)
+            except APIError as error:
+                if error.code == 'uploaddisabled':
+                    pywikibot.error(
+                        'Upload error: Local file uploads are disabled on {}.'
+                        .format(site))
+                elif error.code == 'copyuploadbaddomain' and not download \
+                        and '://' in file_url:
+                    pywikibot.exception()
+                    pywikibot.output('Downloading the file and retry...')
+                    download = True
+                    continue
+                else:
+                    pywikibot.error('Upload error: ', exc_info=True)
+            except Exception:
                 pywikibot.error('Upload error: ', exc_info=True)
-        except Exception:
-            pywikibot.error('Upload error: ', exc_info=True)
-        else:
-            if success:
-                # No warning, upload complete.
-                pywikibot.output('Upload of {} successful.'.format(filename))
-                self._save_counter += 1
-                return filename  # data['filename']
-            pywikibot.output('Upload aborted.')
+            else:
+                if success:
+                    # No warning, upload complete.
+                    pywikibot.output('Upload of {} successful.'
+                                     .format(filename))
+                    self.counter['write'] += 1
+                    return filename  # data['filename']
+                pywikibot.output('Upload aborted.')
+            break
+
         return None
 
-    def skip_run(self):
+    def skip_run(self) -> bool:
         """Check whether processing is to be skipped."""
         # early check that upload is enabled
         if self.target_site.is_uploaddisabled():
@@ -456,7 +464,7 @@ class UploadRobot(BaseBot):
         try:
             for file_url in self.url:
                 self.upload_file(file_url)
-                self._treat_counter += 1
+                self.counter['read'] += 1
         except QuitKeyboardInterrupt:
             pywikibot.output('\nUser quit {} bot run...'
                              .format(self.__class__.__name__))
