@@ -1,10 +1,11 @@
 """Module containing various formatting related utilities."""
 #
-# (C) Pywikibot team, 2015-2021
+# (C) Pywikibot team, 2015-2022
 #
 # Distributed under the terms of the MIT license.
 #
 import math
+import platform
 from string import Formatter
 from typing import Any, Mapping, Sequence
 
@@ -36,7 +37,7 @@ class SequenceOutputter:
     prefix = '\n'
     suffix = '\n'
 
-    def __init__(self, sequence):
+    def __init__(self, sequence) -> None:
         """Create a new instance with a reference to the sequence."""
         super().__init__()
         self.sequence = sequence
@@ -59,7 +60,7 @@ class SequenceOutputter:
             content = ''
         return self.prefix + content + self.suffix
 
-    def output(self):
+    def output(self) -> None:
         """Output the text of the current sequence."""
         output(self.out)
 
@@ -122,6 +123,8 @@ class _ColorFormatter(Formatter):
         return super().vformat(format_string, args, kwargs)
 
 
+@deprecated('New color format pattern like <<color>>colored text<<default>>',
+            since='7.2.0')
 def color_format(text: str, *args, **kwargs) -> str:
     r"""
     Do ``str.format`` without having to worry about colors.
@@ -133,7 +136,14 @@ def color_format(text: str, *args, **kwargs) -> str:
     You may use a variant {color} by assigning a valid color to a named
     parameter color.
 
+    .. deprecated:: 7.2
+       new color format pattern like <<color>>colored text<<default>>
+       may be used instead.
+
     :param text: The format template string
     :return: The formatted string
     """
+    if platform.python_implementation() == 'PyPy' \
+       and isinstance(text, bytes):  # T296830
+        raise TypeError("'text' parameter must be a str not bytes")
     return _ColorFormatter().format(text, *args, **kwargs)
