@@ -3,10 +3,11 @@ A window with a textfield where the user can edit.
 
 Useful for editing the contents of an article.
 
-*New in version 6.1:* Python 3.6 or highter is required.
+.. versionchanged:: 6.1
+   Python 3.6 or highter is required.
 """
 #
-# (C) Pywikibot team, 2003-2021
+# (C) Pywikibot team, 2003-2022
 #
 # Distributed under the terms of the MIT license.
 #
@@ -44,7 +45,7 @@ class TextEditor(ScrolledText):
 
     """
 
-    def __init__(self, master=None, **kwargs):
+    def __init__(self, master=None, **kwargs) -> None:
         """
         Initializer.
 
@@ -99,7 +100,7 @@ class TextEditor(ScrolledText):
                 Theme, 'cursor', fgBg='fg')
         return config
 
-    def add_bindings(self):
+    def add_bindings(self) -> None:
         """Assign key and events bindings to methods."""
         # due to IDLE dependencies, this can't be called from __init__
         # add key and event bindings
@@ -136,46 +137,46 @@ class TextEditor(ScrolledText):
             if keylist:
                 self.event_add(event, *keylist)
 
-    def cut(self, event):
+    def cut(self, event) -> str:
         """Perform cut operation."""
         if self.tag_ranges('sel'):
             self.event_generate('<<Cut>>')
         return 'break'
 
-    def copy(self, event):
+    def copy(self, event) -> str:
         """Perform copy operation."""
         if self.tag_ranges('sel'):
             self.event_generate('<<Copy>>')
         return 'break'
 
-    def paste(self, event):
+    def paste(self, event) -> str:
         """Perform paste operation."""
         self.event_generate('<<Paste>>')
         return 'break'
 
-    def select_all(self, event=None):
+    def select_all(self, event=None) -> str:
         """Perform select all operation."""
         self.tag_add('sel', '1.0', 'end-1c')
         self.mark_set('insert', '1.0')
         self.see('insert')
         return 'break'
 
-    def remove_selection(self, event=None):
+    def remove_selection(self, event=None) -> None:
         """Perform remove operation."""
         self.tag_remove('sel', '1.0', 'end')
         self.see('insert')
 
-    def del_word_left(self, event):
+    def del_word_left(self, event) -> str:
         """Perform delete word (left) operation."""
         self.event_generate('<Meta-Delete>')
         return 'break'
 
-    def del_word_right(self, event=None):
+    def del_word_right(self, event=None) -> str:
         """Perform delete word (right) operation."""
         self.event_generate('<Meta-d>')
         return 'break'
 
-    def find_event(self, event=None):
+    def find_event(self, event=None) -> str:
         """Perform find operation."""
         if not self.tag_ranges('sel'):
             found = self.tag_ranges('found')
@@ -186,17 +187,17 @@ class TextEditor(ScrolledText):
         SearchDialog.find(self)
         return 'break'
 
-    def find_again_event(self, event=None):
+    def find_again_event(self, event=None) -> str:
         """Perform find again operation."""
         SearchDialog.find_again(self)
         return 'break'
 
-    def find_selection_event(self, event=None):
+    def find_selection_event(self, event=None) -> str:
         """Perform find selection operation."""
         SearchDialog.find_selection(self)
         return 'break'
 
-    def replace_event(self, event=None):
+    def replace_event(self, event=None) -> str:
         """Perform replace operation."""
         ReplaceDialog.replace(self)
         return 'break'
@@ -252,7 +253,7 @@ class TextEditor(ScrolledText):
                     self.do_highlight(found[0], found[1])
         return None
 
-    def do_highlight(self, start, end):
+    def do_highlight(self, start, end) -> None:
         """Select and show the text from index start to index end."""
         self.see(start)
         self.tag_remove(tkinter.SEL, '1.0', tkinter.END)
@@ -277,7 +278,7 @@ class EditBoxWindow(tkinter.Frame):
 
     """Edit box window."""
 
-    def __init__(self, parent=None, **kwargs):
+    def __init__(self, parent=None, **kwargs) -> None:
         """Initializer."""
         if parent is None:
             # create a new window
@@ -372,7 +373,7 @@ class EditBoxWindow(tkinter.Frame):
         self.pack()
 
     def edit(self, text: str, jumpIndex: Optional[int] = None,
-             highlight: Optional[str] = None):
+             highlight: Optional[str] = None) -> Optional[str]:
         """
         Provide user with editor to modify text.
 
@@ -381,7 +382,6 @@ class EditBoxWindow(tkinter.Frame):
         :param highlight: each occurrence of this substring will be highlighted
         :return: the modified text, or None if the user didn't save the text
             file in his text editor
-        :rtype: str or None
         """
         self.text = None
         # put given text into our textarea
@@ -404,23 +404,23 @@ class EditBoxWindow(tkinter.Frame):
         self.parent.mainloop()
         return self.text
 
-    def find_all(self, target):
+    def find_all(self, target) -> None:
         """Perform find all operation."""
         self.textfield.insert(tkinter.END, target)
         self.editbox.find_all(target)
 
-    def find(self):
+    def find(self) -> None:
         """Perform find operation."""
         # get text to search for
         s = self.textfield.get()
         if s:
             self.editbox.find_all(s)
 
-    def config_dialog(self, event=None):
+    def config_dialog(self, event=None) -> None:
         """Show config dialog."""
         ConfigDialog(self, 'Settings')
 
-    def pressedOK(self):
+    def pressedOK(self) -> None:
         """
         Perform OK operation.
 
@@ -430,7 +430,7 @@ class EditBoxWindow(tkinter.Frame):
         self.text = self.editbox.get('1.0', tkinter.END)
         self.parent.destroy()
 
-    def debug(self, event=None):
+    def debug(self, event=None) -> str:
         """Call quit() and return 'break'."""
         self.quit()
         return 'break'
@@ -443,7 +443,7 @@ class ListBoxWindow:
 
     # called when user pushes the OK button.
     # closes the window.
-    def pressedOK(self):
+    def pressedOK(self) -> None:
         """
         Perform OK operation.
 
@@ -451,7 +451,7 @@ class ListBoxWindow:
         """
         self.parent.destroy()
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None) -> None:
         """Initializer."""
         if parent is None:
             # create a new window
@@ -494,7 +494,7 @@ class Tkdialog:
 
     """The dialog window for image info."""
 
-    def __init__(self, photo_description, photo, filename):
+    def __init__(self, photo_description, photo, filename) -> None:
         """Initializer."""
         self.root = tkinter.Tk()
         # "%dx%d%+d%+d" % (width, height, xoffset, yoffset)
@@ -582,13 +582,13 @@ class Tkdialog:
         imageTk = ImageTk.PhotoImage(image)
         return imageTk
 
-    def ok_file(self):
+    def ok_file(self) -> None:
         """The user pressed the OK button."""
         self.filename = self.filename_field.get()
         self.photo_description = self.description_field.get(0.0, tkinter.END)
         self.root.destroy()
 
-    def skip_file(self):
+    def skip_file(self) -> None:
         """The user pressed the Skip button."""
         self.skip = True
         self.root.destroy()
