@@ -1,6 +1,6 @@
 """Base for terminal user interfaces."""
 #
-# (C) Pywikibot team, 2003-2021
+# (C) Pywikibot team, 2003-2022
 #
 # Distributed under the terms of the MIT license.
 #
@@ -13,7 +13,7 @@ from typing import Any, Optional, Union
 
 import pywikibot
 from pywikibot import config
-from pywikibot.backports import Sequence
+from pywikibot.backports import Iterable, Sequence, Tuple
 from pywikibot.bot_choice import (
     ChoiceException,
     Option,
@@ -22,13 +22,14 @@ from pywikibot.bot_choice import (
     StandardOption,
 )
 from pywikibot.logging import INFO, INPUT, STDOUT, VERBOSE, WARNING
-from pywikibot.tools import deprecated_args
+from pywikibot.tools import RLock
 from pywikibot.userinterfaces import transliteration
 from pywikibot.userinterfaces._interface_base import ABUIC
 
 
 transliterator = transliteration.transliterator(config.console_encoding)
 
+#: Colors supported by Pywikibot
 colors = [
     'default',
     'black',
@@ -49,35 +50,48 @@ colors = [
     'white',
 ]
 
-_color_pat = '{}|previous'.format('|'.join(colors))
-colorTagR = re.compile('\03{((:?%s);?(:?%s)?)}' % (_color_pat, _color_pat))
+_color_pat = '((:?{0});?(:?{0})?)'.format('|'.join(colors + ['previous']))
+old_colorTagR = re.compile('\03{{{cpat}}}'.format(cpat=_color_pat))
+new_colorTagR = re.compile('<<{cpat}>>'.format(cpat=_color_pat))
 
 
 class UI(ABUIC):
 
     """Base for terminal user interfaces.
 
-    *New in version 6.2:* subclassed from
-    :py:obj:`pywikibot.userinterfaces._interface_base.ABUIC`.
+    .. versionchanged:: 6.2:
+       subclassed from
+       :py:obj:`pywikibot.userinterfaces._interface_base.ABUIC`
     """
 
     split_col_pat = re.compile(r'(\w+);?(\w+)?')
 
-    def __init__(self):
+    def __init__(self) -> None:
         """
         Initialize the UI.
 
         This caches the std-streams locally so any attempts to
         monkey-patch the streams later will not work.
+
+        .. versionchanged:: 7.1
+           memorize original streams
         """
-        self.stdin = sys.stdin
-        self.stdout = sys.stdout
-        self.stderr = sys.stderr
+        # for Windows GUI they can be None under some conditions
+        self.stdin = sys.__stdin__ or sys.stdin
+        self.stdout = sys.__stdout__ or sys.stdout
+        self.stderr = sys.__stderr__ or sys.stderr
+
         self.argv = sys.argv
         self.encoding = config.console_encoding
         self.transliteration_target = config.transliteration_target
+        self.cache = []
+        self.lock = RLock()
 
-    def init_handlers(self, root_logger, default_stream='stderr'):
+    def init_handlers(
+        self,
+        root_logger,
+        default_stream: str = 'stderr'
+    ) -> None:
         """Initialize the handlers for user output.
 
         This method initializes handler(s) for output levels VERBOSE (if
@@ -122,7 +136,7 @@ class UI(ABUIC):
         warnings_logger.addHandler(warning_handler)
 
     def encounter_color(self, color, target_stream):
-        """Handle the next color encountered."""
+        """Abstract method to handle the next color encountered."""
         raise NotImplementedError('The {} class does not support '
                                   'colors.'.format(self.__class__.__name__))
 
@@ -137,15 +151,43 @@ class UI(ABUIC):
         """
         return cls.split_col_pat.search(color).groups()
 
-    def _write(self, text, target_stream):
-        """Optionally encode and write the text to the target stream."""
-        target_stream.write(text)
+    def _write(self, text: str, target_stream) -> None:
+        """Write the text to the target stream.
 
-    def support_color(self, target_stream):
+        sys.stderr and sys.stdout are frozen upon initialization to
+        original streams (which are stored in the sys module as
+        `sys.__stderr__` and `sys.__stdout__`). This works fine except
+        when using `redirect_stderr` or `redirect_stdout` context
+        managers, where values of global `sys.stderr` and `sys.stdout`
+        are temporarily changed to a redirecting `StringIO` stream, in
+        which case writing to the frozen streams (which are still set to
+        `sys.__stderr__` / `sys.__stdout__`) will not write to the
+        redirecting stream as expected. So, we check the target stream
+        against the frozen streams, and then write to the (potentially
+        redirected) `sys.stderr` or `sys.stdout` stream.
+
+        .. versionchanged:: 7.1
+           instead of writing to `target_stream`, dispatch to
+           `sys.stderr` or `sys.stdout`.
+        """
+        if target_stream == self.stderr:
+            sys.stderr.write(text)
+        elif target_stream == self.stdout:
+            sys.stdout.write(text)
+        else:  # pragma: no cover
+            try:
+                out, err = self.stdout.name, self.stderr.name
+            except AttributeError:
+                out, err = self.stdout, self.stderr
+            raise OSError(
+                'Target stream {} is neither stdin ({}) nor stderr ({})'
+                .format(target_stream.name, out, err))
+
+    def support_color(self, target_stream) -> bool:
         """Return whether the target stream does support colors."""
         return False
 
-    def _print(self, text, target_stream):
+    def _print(self, text, target_stream) -> None:
         """Write the text to the target stream handling the colors."""
         colorized = (config.colorized_output
                      and self.support_color(target_stream))
@@ -153,7 +195,14 @@ class UI(ABUIC):
         # Color tags might be cascaded, e.g. because of transliteration.
         # Therefore we need this stack.
         color_stack = ['default']
-        text_parts = colorTagR.split(text) + ['default']
+        old_parts = old_colorTagR.split(text)
+        new_parts = new_colorTagR.split(text)
+        if min(len(old_parts), len(new_parts)) > 1:
+            raise ValueError('Old color format must not be mixed with new '
+                             'color format. Found:\n'
+                             + text.replace('\03', '\\03'))
+        text_parts = old_parts if len(old_parts) > 1 else new_parts
+        text_parts += ['default']
         # match.split() includes every regex group; for each matched color
         # fg_col:b_col, fg_col and bg_col are added to the resulting list.
         len_text_parts = len(text_parts[::4])
@@ -182,15 +231,54 @@ class UI(ABUIC):
 
             if current_color != next_color and colorized:
                 # set the new color, but only if they change
+                # flush first to enable color change on Windows (T283808)
+                target_stream.flush()
                 self.encounter_color(color_stack[-1], target_stream)
 
-    def output(self, text, targetStream=None):
+        # finally flush stream to show the text, required for Windows (T303373)
+        target_stream.flush()
+
+    def output(self, text, targetStream=None) -> None:
+        """Forward text to cache and flush if output is not locked.
+
+        All input methods locks the output to a stream but collect them
+        in cache. They will be printed with next unlocked output call or
+        at termination time.
+
+        .. versionchanged:: 7.0
+           Forward text to cache and flush if output is not locked.
         """
-        Output text to a stream.
+        self.cache_output(text, targetStream=targetStream)
+        if not self.lock.locked():
+            self.flush()
+
+    def flush(self) -> None:
+        """Output cached text.
+
+        .. versionadded:: 7.0
+        """
+        with self.lock:
+            for args, kwargs in self.cache:
+                self.stream_output(*args, **kwargs)
+            self.cache.clear()
+
+    def cache_output(self, *args, **kwargs) -> None:
+        """Put text to cache.
+
+        .. versionadded:: 7.0
+        """
+        with self.lock:
+            self.cache.append((args, kwargs))
+
+    def stream_output(self, text, targetStream=None) -> None:
+        """Output text to a stream.
 
         If a character can't be displayed in the encoding used by the user's
         terminal, it will be replaced with a question mark or by a
         transliteration.
+
+        .. versionadded:: 7.0
+           ``UI.output()`` was renamed to ``UI.stream_output()``
         """
         if config.transliterate:
             # Encode our unicode string in the encoding used by the user's
@@ -245,8 +333,10 @@ class UI(ABUIC):
         # May be overridden by subclass
         return input()
 
-    def input(self, question: str, password: bool = False,
-              default: str = '', force: bool = False) -> str:
+    def input(self, question: str,
+              password: bool = False,
+              default: Optional[str] = '',
+              force: bool = False) -> str:
         """
         Ask the user a question and return the answer.
 
@@ -276,27 +366,29 @@ class UI(ABUIC):
         question += end_marker
 
         # lock stream output
-        # with self.lock: (T282962)
-        if force:
-            self.output(question + '\n')
-            return default
-        # sound the terminal bell to notify the user
-        if config.ring_bell:
-            sys.stdout.write('\07')
-        # TODO: make sure this is logged as well
-        while True:
-            self.output(question + ' ')
-            text = self._input_reraise_cntl_c(password)
-
-            if text is None:
-                continue
-
-            if text:
-                return text
-
-            if default is not None:
+        with self.lock:
+            if force:
+                self.stream_output(question + '\n')
                 return default
 
+            # sound the terminal bell to notify the user
+            if config.ring_bell:
+                sys.stdout.write('\07')
+
+            # TODO: make sure this is logged as well
+            while True:
+                self.stream_output(question + ' ')
+                text = self._input_reraise_cntl_c(password)
+
+                if text is None:
+                    continue
+
+                if text:
+                    return text
+
+                if default is not None:
+                    return default
+
     def _input_reraise_cntl_c(self, password):
         """Input and decode, and re-raise Control-C."""
         try:
@@ -313,9 +405,15 @@ class UI(ABUIC):
             return None  # wrong terminal encoding, T258143
         return text
 
-    def input_choice(self, question: str, options, default: str = None,
+    def input_choice(self, question: str,
+                     options: Union[
+                         Iterable[Union[Tuple[str, str],
+                                        'pywikibot.bot_choice.Option']],
+                         'pywikibot.bot_choice.Option'],
+                     default: Optional[str] = None,
                      return_shortcut: bool = True,
-                     automatic_quit: bool = True, force: bool = False):
+                     automatic_quit: bool = True,
+                     force: bool = False) -> Any:
         """
         Ask the user and returns a value from the options.
 
@@ -329,9 +427,6 @@ class UI(ABUIC):
             Alternatively they may be Option (or subclass) instances or
             ChoiceException instances which have a full option and shortcut
             and will be raised if selected.
-        :type options: iterable containing sequences of length 2 or
-            iterable containing Option instances or ChoiceException as well.
-            Singletons of Option and its subclasses are also accepted.
         :param default: The default answer if no was entered. None to require
             an answer.
         :param return_shortcut: Whether the shortcut or the index in the option
@@ -342,13 +437,12 @@ class UI(ABUIC):
         :return: If return_shortcut the shortcut of options or the value of
             default (if it's not None). Otherwise the index of the answer in
             options. If default is not a shortcut, it'll return -1.
-        :rtype: int (if not return_shortcut), lowercased str (otherwise)
         """
-        def output_option(option, before_question):
+        def output_option(option, before_question) -> None:
             """Print an OutputOption before or after question."""
             if isinstance(option, OutputOption) \
                and option.before_question is before_question:
-                self.output(option.out + '\n')
+                self.stream_output(option.out + '\n')
 
         if force and default is None:
             raise ValueError('With no default option it cannot be forced')
@@ -373,24 +467,24 @@ class UI(ABUIC):
         handled = False
 
         # lock stream output
-        # with self.lock: (T282962)
-        while not handled:
-            for option in options:
-                output_option(option, before_question=True)
-            output = Option.formatted(question, options, default)
-            if force:
-                self.output(output + '\n')
-                answer = default
-            else:
-                answer = self.input(output) or default
-            # something entered or default is defined
-            if answer:
-                for index, option in enumerate(options):
-                    if option.handled(answer):
-                        answer = option.result(answer)
-                        output_option(option, before_question=False)
-                        handled = option.stop
-                        break
+        with self.lock:
+            while not handled:
+                for option in options:
+                    output_option(option, before_question=True)
+                output = Option.formatted(question, options, default)
+                if force:
+                    self.stream_output(output + '\n')
+                    answer = default
+                else:
+                    answer = self.input(output) or default
+                # something entered or default is defined
+                if answer:
+                    for index, option in enumerate(options):
+                        if option.handled(answer):
+                            answer = option.result(answer)
+                            output_option(option, before_question=False)
+                            handled = option.stop
+                            break
 
         if isinstance(answer, ChoiceException):
             raise answer
@@ -411,34 +505,36 @@ class UI(ABUIC):
         :return: Return a single Sequence entry.
         """
         # lock stream output
-        # with self.lock: (T282962)
-        if not force:
-            line_template = '{{0: >{}}}: {{1}}'.format(len(str(len(answers))))
-            for i, entry in enumerate(answers, start=1):
-                pywikibot.output(line_template.format(i, entry))
+        with self.lock:
+            if not force:
+                line_template = '{{0: >{}}}: {{1}}\n'.format(
+                    len(str(len(answers))))
+                for i, entry in enumerate(answers, start=1):
+                    self.stream_output(line_template.format(i, entry))
 
-        while True:
-            choice = self.input(question, default=default, force=force)
+            while True:
+                choice = self.input(question, default=default, force=force)
 
-            try:
-                choice = int(choice) - 1
-            except (TypeError, ValueError):
-                if choice in answers:
-                    return choice
-                choice = -1
+                try:
+                    choice = int(choice) - 1
+                except (TypeError, ValueError):
+                    if choice in answers:
+                        return choice
+                    choice = -1
 
-            # User typed choice number
-            if 0 <= choice < len(answers):
-                return answers[choice]
+                # User typed choice number
+                if 0 <= choice < len(answers):
+                    return answers[choice]
 
-            if force:
-                raise ValueError('Invalid value "{}" for default during force.'
-                                 .format(default))
+                if force:
+                    raise ValueError(
+                        'Invalid value "{}" for default during force.'
+                        .format(default))
 
-            pywikibot.error('Invalid response')
+                self.stream_output('Error: Invalid response\n')
 
     def editText(self, text: str, jumpIndex: Optional[int] = None,
-                 highlight: Optional[str] = None):
+                 highlight: Optional[str] = None) -> Optional[str]:
         """Return the text as edited by the user.
 
         Uses a Tkinter edit box because we don't have a console editor
@@ -448,7 +544,6 @@ class UI(ABUIC):
         :param highlight: each occurrence of this substring will be highlighted
         :return: the modified text, or None if the user didn't save the text
             file in his text editor
-        :rtype: str or None
         """
         try:
             from pywikibot.userinterfaces import gui
@@ -458,10 +553,6 @@ class UI(ABUIC):
         editor = gui.EditBoxWindow()
         return editor.edit(text, jumpIndex=jumpIndex, highlight=highlight)
 
-    def argvu(self):
-        """Return copy of argv."""
-        return list(self.argv)
-
 
 class TerminalHandler(logging.StreamHandler):
 
@@ -477,8 +568,7 @@ class TerminalHandler(logging.StreamHandler):
     # create a class-level lock that can be shared by all instances
     sharedlock = threading.RLock()
 
-    @deprecated_args(strm='stream')
-    def __init__(self, UI, stream=None):
+    def __init__(self, UI, stream=None) -> None:
         """Initialize the handler.
 
         If stream is not specified, sys.stderr is used.
@@ -486,7 +576,7 @@ class TerminalHandler(logging.StreamHandler):
         super().__init__(stream=stream)
         self.UI = UI
 
-    def createLock(self):
+    def createLock(self) -> None:
         """Acquire a thread lock for serializing access to the underlying I/O.
 
         Replace Handler's instance-specific lock with the shared
@@ -495,7 +585,7 @@ class TerminalHandler(logging.StreamHandler):
         """
         self.lock = TerminalHandler.sharedlock
 
-    def emit(self, record):
+    def emit(self, record) -> None:
         """Emit the record formatted to the output."""
         self.flush()
         if record.name == 'py.warnings':
@@ -519,7 +609,7 @@ class MaxLevelFilter(logging.Filter):
 
     """
 
-    def __init__(self, level=None):
+    def __init__(self, level=None) -> None:
         """Initializer."""
         self.level = level
 
